@@ -1,12 +1,18 @@
-
 import logging
+import multiprocessing as mp
 import os
 import sys
 import tempfile
+
+from datetime import datetime
 from pathlib import Path
 
+import bakta.constants as bc
+
+
 log = logging.getLogger('CONFIG')
 
+
 # runtime configurations
 env = os.environ.copy()
 threads = None
@@ -37,6 +43,8 @@ locus = None
 locus_tag = None
 gram = None
 replicons = None
+compliant = None
+user_proteins = None
 
 # workflow configuration
 skip_trna = None
@@ -50,15 +58,23 @@ skip_sorf = None
 skip_gap = None
 skip_ori = None
 
+run_start = datetime.now()
+run_end = None
+
 
 def setup(args):
     """Test environment and build a runtime configuration."""
     # runtime configurations
     global env, threads, verbose
+    env['BLAST_USAGE_REPORT'] = 'false'  # prevent BLAST from contacting NCBI
+
     threads = args.threads
     if(threads <= 0):
-        log.error("wrong argument for 'threads' parameter! threads=%s", threads)
-        sys.exit(f"ERROR: wrong argument ({threads}) for 'threads' parameter! Value must be larger than 0")
+        log.error("wrong argument for 'threads' parameter! threads=%i", threads)
+        sys.exit(f"ERROR: wrong argument ({threads}) for 'threads' parameter! Value must be larger than 0.")
+    elif(threads > mp.cpu_count()):
+        log.error("wrong argument for 'threads' parameter! More threads requested than available: requested=%i, available=%i", threads, mp.cpu_count())
+        sys.exit(f"ERROR: wrong argument ({threads}) for 'threads' parameter! More threads requested ({threads}) than available ({mp.cpu_count()}).")
     log.info('threads=%i', threads)
     verbose = args.verbose
     log.info('verbose=%s', verbose)
@@ -102,16 +118,16 @@ def setup(args):
             log.error('unvalid database path: type=base-dir, path=%s', db_tmp_path)
             sys.exit('ERROR: database neither provided nor auto-detected!\nPlease, download the mandatory db and provide it via either the --db parameter, a BAKTA_DB environment variable or copy it into the Bakta base directory.\nFor further information please read the readme.md')
 
-    if(args.tmp_dir):
+    if(args.tmp_dir is not None):
         tmp_path = Path(args.tmp_dir)
         if(not tmp_path.exists()):
             log.debug('dedicated temp dir does not exist! tmp-dir=%s', tmp_path)
             sys.exit(f'ERROR: dedicated temporary directory ({tmp_path}) does not exist!')
         else:
             log.info('use dedicated temp dir: path=%s', tmp_path)
-            tmp_path = Path(tempfile.mkdtemp(dir=str(tmp_path)))
+            tmp_path = Path(tempfile.mkdtemp(dir=str(tmp_path))).resolve()
     else:
-        tmp_path = Path(tempfile.mkdtemp())
+        tmp_path = Path(tempfile.mkdtemp()).resolve()
     log.info('tmp-path=%s', tmp_path)
 
     try:
@@ -137,24 +153,38 @@ def setup(args):
     # organism configurations
     global genus, species, strain, plasmid, taxon
     genus = args.genus
-    if(genus):
-        genus = genus.capitalize()
+    if(genus is not None):
+        genus = genus.strip()
+        if(genus == ''):
+            log.error("Empty 'genus' parameter! genus=%s", genus)
+            sys.exit(f"ERROR: empty 'genus' parameter!")
+        else:
+            genus = genus.capitalize()
     log.info('genus=%s', genus)
     species = args.species
-    if(species):
-        species = species.lower()
+    if(species is not None):
+        species = species.strip()
+        if(species == ''):
+            log.error("Empty 'species' parameter! species=%s", species)
+            sys.exit(f"ERROR: empty 'species' parameter!")
+        else:
+            species = species.lower()
     log.info('species=%s', species)
     strain = args.strain
+    if(strain is not None):
+        strain = strain.strip()
+        if(strain == ''):
+            log.error("Empty 'strain' parameter! strain=%s", species)
+            sys.exit(f"ERROR: empty 'strain' parameter!")
     log.info('strain=%s', strain)
     plasmid = args.plasmid
     log.info('plasmid=%s', plasmid)
-    taxon = f'{genus} {species} {strain}'
-    taxon = ' '.join(taxon.replace('None', '').split())
+    taxon = ' '.join([t for t in [genus, species, strain] if t is not None])
     if(taxon == ''):
         taxon = None
 
     # annotation configurations
-    global complete, prodigal_tf, translation_table, keep_contig_headers, locus, locus_tag, gram, replicons
+    global complete, prodigal_tf, translation_table, keep_contig_headers, locus, locus_tag, gram, replicons, compliant, user_proteins
     complete = args.complete
     log.info('complete=%s', complete)
     prodigal_tf = args.prodigal_tf
@@ -174,9 +204,39 @@ def setup(args):
     log.info('translation_table=%s', translation_table)
     gram = args.gram
     log.info('gram=%s', gram)
+    compliant = args.compliant
+    log.info('compliant=%s', compliant)
+    if(compliant):
+        min_contig_length = 200
+        log.info('compliant mode! min_contig_length=%s', min_contig_length)
     locus = args.locus
+    if(locus is not None):
+        if(locus == ''):
+            log.error("Empty 'locus' parameter! locus=%s", locus)
+            sys.exit(f"ERROR: empty 'locus' parameter!")
+        if(' ' in locus):
+            log.error("Whitespace character in 'locus' parameter! locus=%s", locus)
+            sys.exit(f"ERROR: whitespace character ({locus}) in 'locus' parameter!")
+        if(bc.RE_INSDC_ID_PREFIX.fullmatch(locus) is None):
+            log.error("Invalid 'locus' parameter! locus=%s", locus)
+            sys.exit(f"ERROR: invalid 'locus' parameter ({locus})!\nLocus prefixes must contain between 1 and 20 alphanumeric or '-_' characters.")
     log.info('locus=%s', locus)
     locus_tag = args.locus_tag
+    if(locus_tag is not None):
+        if(locus_tag == ''):
+            log.error("Empty 'locus-tag' parameter! locus=%s", locus_tag)
+            sys.exit(f"ERROR: empty 'locus-tag' parameter!")
+        if(' ' in locus_tag):
+            log.error("Whitespace character in 'locus-tag' parameter! locus-tag=%s", locus_tag)
+            sys.exit(f"ERROR: whitespace character ({locus_tag}) in 'locus-tag' parameter!")
+        if(compliant):
+            if(bc.RE_INSDC_LOCUSTAG_PREFIX.fullmatch(locus_tag) is None):
+                log.error("INSDC-incompliant 'locus-tag' parameter! locus-tag=%s", locus_tag)
+                sys.exit(f"ERROR: INSDC-incompliant 'locus-tag' parameter ({locus_tag})!\nINSDC Locus tag prefixes must contain between 3 and 12 alphanumeric uppercase characters and start with a letter.")
+        else:
+            if(bc.RE_LOCUSTAG_PREFIX.fullmatch(locus_tag) is None):
+                log.error("Invalid 'locus-tag' parameter! locus-tag=%s", locus_tag)
+                sys.exit(f"ERROR: invalid 'locus-tag' parameter ({locus_tag})!\nLocus tag prefixes must contain between 1 and 24 alphanumeric characters or '_.-' signs.")
     log.info('locus-tag=%s', locus_tag)
     keep_contig_headers = args.keep_contig_headers
     log.info('keep_contig_headers=%s', keep_contig_headers)
@@ -193,7 +253,20 @@ def setup(args):
             log.error('provided replicon file not valid! path=%s', replicons)
             sys.exit(f'ERROR: replicon table file ({replicons}) not valid!')
     log.info('replicon-table=%s', replicons)
-    
+    user_proteins = args.proteins
+    if(user_proteins is not None):
+        try:
+            if(user_proteins == ''):
+                raise ValueError('File path argument must be non-empty')
+            user_proteins_path = Path(args.proteins).resolve()
+            check_readability('user proteins', user_proteins_path)
+            check_content_size('user proteins', user_proteins_path)
+            user_proteins = user_proteins_path
+        except:
+            log.error('provided user proteins file not valid! path=%s', user_proteins)
+            sys.exit(f'ERROR: user proteins file ({user_proteins}) not valid!')
+    log.info('user-proteins=%s', user_proteins)
+
     # workflow configurations
     global skip_trna, skip_tmrna, skip_rrna, skip_ncrna, skip_ncrna_region, skip_crispr, skip_cds, skip_sorf, skip_gap, skip_ori
     skip_trna = args.skip_trna
