@@ -1,4 +1,4 @@
-
+import hashlib
 import json
 import logging
 import os
@@ -8,55 +8,21 @@ import stat
 import sys
 import tarfile
 import tempfile
-import hashlib
 
 from pathlib import Path
+
+from alive_progress import alive_bar
 import requests
 
 import bakta
 import bakta.constants as bc
 import bakta.utils as bu
 
+
 log = logging.getLogger('DB')
 
 
-def check(db_path):
-    """Check if database directory exists, is accessible and contains necessary files."""
-
-    if(db_path is None):
-        log.error('directory neither provided nor detected!')
-        sys.exit('ERROR: database directory not provided nor detected! Please provide a valid path to the database directory.')
-
-    if(not os.access(str(db_path), os.R_OK & os.X_OK)):
-        log.error('directory (%s) not readable/accessible!', db_path)
-        sys.exit(f'ERROR: database directory ({db_path}) not readable/accessible!')
-
-    version_path = db_path.joinpath('version.json')
-    if(not os.access(str(version_path), os.R_OK) or not version_path.is_file()):
-        log.error('version file not readable!')
-        sys.exit('ERROR: database version file (version.json) not readable!')
-    
-    try:
-        with version_path.open() as fh:
-            db_info = json.load(fh)
-    except:
-        log.exception('could not parse database version file!')
-        sys.exit('ERROR: could not parse database version file!')
-    
-    for key in ['date', 'major', 'minor']:
-        if(key not in db_info):
-            log.error('wrong db version info file content! missed key=%s', key)
-            sys.exit(f"ERROR: wrong db version info file format! Missed key '{key}' in JSON structure.")
-    
-    log.info('detected: major=%i, minor=%i, date=%s', db_info['major'], db_info['minor'], db_info['date'])
-    if(db_info['major'] < bakta.__db_schema_version__):
-        log.error('wrong database version detected! required=%i, detected=%i', bakta.__db_schema_version__, db_info['major'])
-        sys.exit(f"ERROR: wrong database version detected!\nBakta version {bakta.__version__} requires database version {bakta.__db_schema_version__}.x, but {db_info['major']}.{db_info['minor']} was detected. Please, update the database from https://doi.org/10.5281/zenodo.4247253")
-    elif(db_info['major'] > bakta.__db_schema_version__):
-        log.error('wrong database version detected! required=%i, detected=%i', bakta.__db_schema_version__, db_info['major'])
-        sys.exit(f"ERROR: wrong database version detected!\nBakta version {bakta.__version__} requires database version {bakta.__db_schema_version__}.x, but {db_info['major']}.{db_info['minor']} was detected. Please, update Bakta or download a compatible database version from https://doi.org/10.5281/zenodo.4247253")
-
-    file_names = [
+FILE_NAMES = [
         'antifam.h3f',
         'antifam.h3i',
         'antifam.h3m',
@@ -86,7 +52,44 @@ def check(db_path):
         'sorf.dmnd'
     ]
 
-    for file_name in file_names:
+
+def check(db_path):
+    """Check if database directory exists, is accessible and contains necessary files."""
+
+    if(db_path is None):
+        log.error('directory neither provided nor detected!')
+        sys.exit('ERROR: database directory not provided nor detected! Please provide a valid path to the database directory.')
+
+    if(not os.access(str(db_path), os.R_OK & os.X_OK)):
+        log.error('directory (%s) not readable/accessible!', db_path)
+        sys.exit(f'ERROR: database directory ({db_path}) not readable/accessible!')
+
+    version_path = db_path.joinpath('version.json')
+    if(not os.access(str(version_path), os.R_OK) or not version_path.is_file()):
+        log.error('version file not readable!')
+        sys.exit('ERROR: database version file (version.json) not readable!')
+
+    try:
+        with version_path.open() as fh:
+            db_info = json.load(fh)
+    except:
+        log.exception('could not parse database version file!')
+        sys.exit('ERROR: could not parse database version file!')
+
+    for key in ['date', 'major', 'minor']:
+        if(key not in db_info):
+            log.error('wrong db version info file content! missed key=%s', key)
+            sys.exit(f"ERROR: wrong db version info file format! Missed key '{key}' in JSON structure.")
+
+    log.info('detected: major=%i, minor=%i, date=%s', db_info['major'], db_info['minor'], db_info['date'])
+    if(db_info['major'] < bakta.__db_schema_version__):
+        log.error('wrong database version detected! required=%i, detected=%i', bakta.__db_schema_version__, db_info['major'])
+        sys.exit(f"ERROR: wrong database version detected!\nBakta version {bakta.__version__} requires database version {bakta.__db_schema_version__}.x, but {db_info['major']}.{db_info['minor']} was detected. Please, update the database from https://doi.org/10.5281/zenodo.4247253")
+    elif(db_info['major'] > bakta.__db_schema_version__):
+        log.error('wrong database version detected! required=%i, detected=%i', bakta.__db_schema_version__, db_info['major'])
+        sys.exit(f"ERROR: wrong database version detected!\nBakta version {bakta.__version__} requires database version {bakta.__db_schema_version__}.x, but {db_info['major']}.{db_info['minor']} was detected. Please, update Bakta or download a compatible database version from https://doi.org/10.5281/zenodo.4247253")
+
+    for file_name in FILE_NAMES:
         path = db_path.joinpath(file_name)
         if(not os.access(str(path), os.R_OK) or not path.is_file()):
             log.error('file not readable! file=%s', file_name)
@@ -110,22 +113,22 @@ def download(db_url, tarball_path):
     try:
         with tarball_path.open('wb') as fh_out, requests.get(db_url, stream=True) as resp:
             total_length = resp.headers.get('content-length')
-            if total_length is None: # no content length header
-                fh_out.write(resp.content)
+            if(total_length is None):  # no content length header
+                with alive_bar() as bar:
+                    for data in resp.iter_content(chunk_size=1024*1024):
+                        fh_out.write(data)
+                        bar()
             else:
-                dl = 0
-                total_length = int(total_length)
-                for data in resp.iter_content(chunk_size=4096):
-                    dl += len(data)
-                    fh_out.write(data)
-                    done = int(50 * dl / total_length)
-                    sys.stdout.write(f"\r\t[{'=' * done}{' ' * (50-done)}]\t{dl/(1024*1024*1024):1.3f} of {total_length/(1024*1024*1024):1.1f} Gb")
-                    sys.stdout.flush()
+                total_length = int(int(total_length)/1024)  # length in Kb
+                with alive_bar(total=total_length) as bar:
+                    for data in resp.iter_content(chunk_size=1024*1024):
+                        fh_out.write(data)
+                        bar(incr=len(data)/1024)
     except IOError:
         sys.exit(f'ERROR: Could not download file from Zenodo! url={db_url}, path={tarball_path}')
 
 
-def calc_md5_sum(tarball_path, buffer_size=8192):
+def calc_md5_sum(tarball_path, buffer_size=1024*1024):
     md5 = hashlib.md5()
     with tarball_path.open('rb') as fh:
         data = fh.read(buffer_size)
@@ -149,28 +152,31 @@ def main():
     group_runtime = parser.add_argument_group('Runtime & auxiliary options')
     group_runtime.add_argument('--help', '-h', action='help', help='Show this help message and exit')
     group_runtime.add_argument('--version', '-V', action='version', version=f'%(prog)s {bakta.__version__}')
-    
+
     subparsers = parser.add_subparsers(dest='subcommand', help='sub-command help')
-    subparsers.add_parser('list', help='List available database versions')  #  add list sub-command options
-    
-    parser_download = subparsers.add_parser('download', help='Download a database')  #  add download sub-command options
+    parser_list = subparsers.add_parser('list', help='List available database versions')  # add list sub-command options
+    parser_list.add_argument('--all', action='store_true', help='Show all versions including incompatible')
+
+    parser_download = subparsers.add_parser('download', help='Download a database')  # add download sub-command options
     parser_download.add_argument('--output', '-o', action='store', default=Path.cwd(), help='output directory (default = current working directory)')
     parser_download.add_argument('--minor', '-n', action='store', type=int, default=0, help='Database minor version (default = most recent db minor version)')
-    
-    parser_update = subparsers.add_parser('update', help='Update an existing database to the most recent compatible version')  #  add download sub-command options
+
+    parser_update = subparsers.add_parser('update', help='Update an existing database to the most recent compatible version')  # add download sub-command options
     parser_update.add_argument('--db', '-d', action='store', default=None, help='Current database path (default = <bakta_path>/db). Can also be provided as BAKTA_DB environment variable.')
     parser_update.add_argument('--tmp-dir', '-t', action='store', dest='tmp_dir', default=Path.cwd(), help='Temporary directory to download & extract (default = current working directory)')
-    
+
     args = parser.parse_args()
     if(args.subcommand == 'list'):
+        print(f'Required database schema version: {bakta.__db_schema_version__}\n')
         versions = fetch_db_versions()
+        if(not args.all):
+            versions = [v for v in versions if v['major'] == bakta.__db_schema_version__]
+
         print('Available DB versions:')
         for v in sorted(versions, key=lambda v: (v['major'], v['minor'])):
             print(f"{v['major']}.{v['minor']}\t{v['date']}\t{v['doi']}")
     elif(args.subcommand == 'download'):
-
         bu.test_dependency(bu.DEPENDENCY_AMRFINDERPLUS)
-
         try:
             output_path = Path(args.output)
             if(not output_path.exists()):
@@ -183,14 +189,14 @@ def main():
         except:
             sys.exit(f'ERROR: could not resolve or create output directory ({args.output})!')
 
-        print(f'fetch DB versions...')
+        print('fetch DB versions...')
         versions = fetch_db_versions()
         compatible_versions = [v for v in versions if v['major'] == bakta.__db_schema_version__]
         if(len(compatible_versions) == 0):
             sys.exit(f'Error: no compatible version available for current major db version {bakta.__db_schema_version__}')
         else:
             print(f'\t... compatible DB versions: {len(compatible_versions)}')
-        
+
         required_version = None
         if(args.minor > 0):
             for v in versions:
@@ -200,9 +206,9 @@ def main():
             if(required_version is None):
                 sys.exit(f"requested DB minor version {args.minor} is not available. Please use 'bakta_db list' to get a list of available DB versions")
         else:
-            compatible_sorted = sorted(compatible_versions, key=lambda v:v['minor'], reverse=True)
+            compatible_sorted = sorted(compatible_versions, key=lambda v: v['minor'], reverse=True)
             required_version = compatible_sorted[0]
-        
+
         tarball_path = output_path.joinpath('db.tar.gz')
         db_url = f"https://zenodo.org/record/{required_version['record']}/files/db.tar.gz"
         print(f"download database: v{required_version['major']}.{required_version['minor']}, {required_version['date']}, DOI: {required_version['doi']}, URL: {db_url}...")
@@ -215,7 +221,7 @@ def main():
             print(f'\t...database file OK: {md5_sum}')
         else:
             sys.exit(f"Error: corrupt database file! MD5 should be '{required_version['md5']}' but is '{md5_sum}'")
-        
+
         print(f'extract DB tarball: file={tarball_path}, output={output_path}')
         untar(tarball_path, output_path)
         tarball_path.unlink()
@@ -231,15 +237,20 @@ def main():
         print(f"\tDOI: {required_version['doi']}")
         print(f'\tpath: {db_path}')
 
-        print(f'update AMRFinderPlus database...')
-        update_amrfinderplus_db()
+        try:
+            db_path.chmod(stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR | stat.S_IRGRP | stat.S_IWGRP | stat.S_IXGRP | stat.S_IROTH | stat.S_IXOTH)  # set write permissions on old (existing) directory with updated content
+            for db_file_path in db_path.iterdir():
+                db_file_path.chmod(stat.S_IRUSR | stat.S_IRGRP | stat.S_IROTH)
+        except:
+            sys.exit(f'ERROR: cannot set read|execute permissions on new database! path={db_path}, owner={db_path.owner()}, group={db_path.group()}, permissions={oct(db_path.stat().st_mode )[-3:]}')
+
+        print('update AMRFinderPlus database...')
+        update_amrfinderplus_db(db_path)
         print('\t... done')
 
         print(f"\nRun Bakta using '--db {db_path}' or set a BAKTA_DB environment variable: 'export BAKTA_DB={db_path}'")
     elif(args.subcommand == 'update'):
-
         bu.test_dependency(bu.DEPENDENCY_AMRFINDERPLUS)
-        
         env = os.environ.copy()
         if(args.db):
             db_dir = args.db
@@ -271,7 +282,7 @@ def main():
                 print(f'database detected in base-dir: path={db_old_path}')
             else:
                 sys.exit('ERROR: database neither auto-detected nor provided!\nPlease, download the mandatory db and provide it either via the --db parameter, via a BAKTA_DB environment variable or copy it into the Bakta base directory.\nFor further information please read the readme.md')
-        
+
         if(args.tmp_dir):
             tmp_path = Path(args.tmp_dir)
             if(not tmp_path.exists()):
@@ -285,15 +296,15 @@ def main():
 
         db_old_info = check(db_old_path)
         print(f"existing database: v{db_old_info['major']}.{db_old_info['minor']}")
-        print(f'fetch DB versions...')
+        print('fetch DB versions...')
         versions = fetch_db_versions()
         compatible_versions = [v for v in versions if v['major'] == bakta.__db_schema_version__]
         if(len(compatible_versions) == 0):
             sys.exit(f'Error: no compatible version available for current major db version {bakta.__db_schema_version__}')
         else:
             print(f'\t... compatible DB versions: {len(compatible_versions)}')
-        
-        compatible_sorted = sorted(compatible_versions, key=lambda v:v['minor'], reverse=True)
+
+        compatible_sorted = sorted(compatible_versions, key=lambda v: v['minor'], reverse=True)
         if(compatible_sorted[0]['minor'] <= db_old_info['minor']):
             print(f"Database version {db_old_info['major']}.{db_old_info['minor']} is up-to-date")
             sys.exit()
@@ -311,7 +322,7 @@ def main():
             print(f'\t...database file OK: {md5_sum}')
         else:
             sys.exit(f"Error: corrupt database file! MD5 should be '{required_version['md5']}' but is '{md5_sum}'")
-        
+
         print(f'extract DB tarball: file={tarball_path}, output-directory={tmp_path}')
         untar(tarball_path, tmp_path)
         tarball_path.unlink()
@@ -326,11 +337,14 @@ def main():
         print(f"\tversion: {required_version['major']}.{required_version['minor']}")
         print(f"\tDOI: {required_version['doi']}")
         print(f'\tpath: {db_new_path}')
-        print(f'remove old database...')
+        print('remove old database...')
         try:
             db_old_path.chmod(stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR)  # set write permissions on old directory
             for db_old_file_path in db_old_path.iterdir():
-                db_old_file_path.chmod(stat.S_IRUSR | stat.S_IWUSR)
+                if(db_old_file_path.is_dir()):
+                    db_old_file_path.chmod(stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR)
+                else:    
+                    db_old_file_path.chmod(stat.S_IRUSR | stat.S_IWUSR)
         except:
             sys.exit(f'ERROR: cannot set read|write|execute permissions on old database! path={db_old_path}, owner={db_old_path.owner()}, group={db_old_path.group()}, permissions={oct(db_old_path.stat().st_mode )[-3:]}')
         try:
@@ -338,7 +352,7 @@ def main():
         except:
             sys.exit(f'ERROR: cannot remove old database! path={db_old_path}, owner={db_old_path.owner()}, group={db_old_path.group()}, permissions={oct(db_old_path.stat().st_mode )[-3:]}')
         db_old_path.mkdir()
-        
+
         try:
             db_new_path.chmod(stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR)  # set write permissions on db_new_path directory
             for db_new_file_path in db_new_path.iterdir():
@@ -352,30 +366,32 @@ def main():
         except:
             sys.exit(f'ERROR: cannot move new database to existing path! new-path={db_new_path}, existing-path={db_old_path.parent}')
         shutil.rmtree(tmp_path)
-        
+
         try:
-            db_old_path.chmod(stat.S_IRUSR | stat.S_IXUSR | stat.S_IRGRP | stat.S_IXGRP | stat.S_IROTH | stat.S_IXOTH)  # set write permissions on old (existing) directory with updated content
+            db_old_path.chmod(stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR | stat.S_IRGRP | stat.S_IWGRP | stat.S_IXGRP | stat.S_IROTH | stat.S_IXOTH)  # set write permissions on old (existing) directory with updated content
             for db_old_file_path in db_old_path.iterdir():
                 db_old_file_path.chmod(stat.S_IRUSR | stat.S_IRGRP | stat.S_IROTH)
         except:
             sys.exit(f'ERROR: cannot set read(|execute) permissions on new database! path={db_old_path}, owner={db_old_path.owner()}, group={db_old_path.group()}, permissions={oct(db_old_path.stat().st_mode )[-3:]}')
-        
+
         print('\t... done')
-        
-        print(f'update AMRFinderPlus database...')
-        update_amrfinderplus_db()
+
+        print('update AMRFinderPlus database...')
+        update_amrfinderplus_db(db_old_path)
         print('\t... done')
-        
+
         print(f"\nRun Bakta using '--db {db_old_path}' or set a BAKTA_DB environment variable: 'export BAKTA_DB={db_old_path}'")
     else:
         parser.print_help()
         sys.exit('Error: no subcommand provided!')
 
 
-def update_amrfinderplus_db():
+def update_amrfinderplus_db(db_path):
+    amrfinderplus_db_path = db_path.joinpath('amrfinderplus-db')
     cmd = [
-        'amrfinder',
-        '-u'
+        'amrfinder_update',
+        '--database', str(amrfinderplus_db_path),
+        '--force_update'
     ]
     log.debug('cmd=%s', cmd)
     proc = sp.run(
@@ -387,7 +403,7 @@ def update_amrfinderplus_db():
     if(proc.returncode != 0):
         log.debug('stdout=\'%s\', stderr=\'%s\'', proc.stdout, proc.stderr)
         log.warning('AMRFinderPlus failed! amrfinder-error-code=%d', proc.returncode)
-        sys.exit(f"ERROR: AMRFinderPlus failed! command: 'amrfinder -u', error code: {proc.returncode}")
+        sys.exit(f"ERROR: AMRFinderPlus failed! command: 'amrfinder_update --force_update --database {amrfinderplus_db_path}', error code: {proc.returncode}")
 
 
 if __name__ == '__main__':
