@@ -1,21 +1,53 @@
 
 import logging
+import re
 
+import bakta.config as cfg
 import bakta.constants as bc
+import bakta.io.insdc as insdc
+
 
 log = logging.getLogger('ANNOTATION')
 
 
+RE_MULTIWHITESPACE = re.compile(r'\s{2,}')
+
+RE_PROTEIN_CONTIG = re.compile(r'\s+contig\s*', flags=re.IGNORECASE)
+RE_PROTEIN_HOMOLOG = re.compile(r'\shomolog(?: (\d+))?', flags=re.IGNORECASE)
+RE_PROTEIN_PUTATIVE = re.compile(r'(potential|possible|probable|predicted)', flags=re.IGNORECASE)
+RE_PROTEIN_NODE = re.compile(r'NODE_', flags=re.IGNORECASE)
+RE_PROTEIN_POTENTIAL_CONTIG_NAME = re.compile(r'(genome|shotgun)', flags=re.IGNORECASE)
+RE_PROTEIN_DOMAIN_CONTAINING = re.compile(r'domain-containing protein', flags=re.IGNORECASE)
+RE_PROTEIN_NO_LETTERS = re.compile(r'[^A-Za-z]')
+RE_PROTEIN_SUSPECT_CHARS = re.compile(r'[.@=?%]')
+RE_PROTEIN_PERIOD_SEPARATOR = re.compile(r'([a-zA-Z0-9]+)\.([a-zA-Z0-9]+)')
+RE_DOMAIN_OF_UNKNOWN_FUCTION = re.compile(r'(DUF\d{3,4})', flags=re.IGNORECASE)
+RE_UNCHARACTERIZED_PROTEIN_FAMILY = re.compile(r'(UPF\d{3,4})', flags=re.IGNORECASE)
+RE_PROTEIN_WEIGHT = re.compile(r' [0-9]+(?:\.[0-9]+)? k?da ', flags=re.IGNORECASE)
+RE_PROTEIN_SYMBOL = re.compile(r'[A-Z][a-z]{2}[A-Z][0-9]?')
+
+RE_GENE_CAPITALIZED = re.compile(r'^[A-Z].+', flags=re.DOTALL)
+RE_GENE_SUSPECT_CHARS = re.compile(r'[\?]', flags=re.DOTALL)
+RE_GENE_SYMBOL = re.compile(r'[a-z]{3}[A-Z][0-9]?')
+
+
 def combine_annotation(feature):
     ups = feature.get('ups', None)
     ips = feature.get('ips', None)
     psc = feature.get('psc', None)
+    pscc = feature.get('pscc', None)
     expert = feature.get('expert', None)
 
     gene = None
     product = None
     db_xrefs = set()
-    if(psc):
+    if(pscc):
+        pscc_product = pscc.get('product', None)
+        if(pscc_product):
+            product = pscc_product
+        for db_xref in pscc['db_xrefs']:
+            db_xrefs.add(db_xref)
+    if(psc and psc.get('valid', True)):
         psc_gene = psc.get('gene', None)
         if(psc_gene):
             gene = psc_gene
@@ -41,27 +73,34 @@ def combine_annotation(feature):
         for expert_system, expert_hit in expert.items():
             expert_rank = expert_hit['rank']
             if(expert_rank > rank):
-                expert_gene = expert_hit.get('gene', None)
-                if(expert_gene):
-                    gene = expert_gene
-                expert_product = expert_hit.get('product', None)
-                if(expert_product):
-                    product = expert_product
+                gene = expert_hit.get('gene', None)
+                product = expert_hit.get('product', None)
                 expert_db_xrefs = expert_hit.get('db_xrefs', None)
                 if(expert_db_xrefs):
                     for expert_db_xref in expert_db_xrefs:
                         db_xrefs.add(expert_db_xref)
                 rank = expert_rank
-    
+
+    feature['gene'] = gene
     if(gene):
-        feature['gene'] = gene
-    
+        revise_cds_gene_symbol(feature)
+
     if(product):
         feature['product'] = product
+        revise_cds_product(feature)
+        if(cfg.compliant and not feature.get('hypothetical', False)):
+            insdc.revise_product_insdc(feature)
     else:
         feature['product'] = bc.HYPOTHETICAL_PROTEIN
         feature['hypothetical'] = True
-    
+
+    if(gene and feature.get('hypothetical', False)):
+        feature['gene'] = None
+        log.info(
+            'remove gene symbol of hypothetical protein: contig=%s, start=%i, stop=%i, strand=%s, gene=%s',
+            feature['contig'], feature['start'], feature['stop'], feature['strand'], gene
+        )
+
     feature['db_xrefs'] = sorted(list(db_xrefs))
 
 
@@ -99,7 +138,7 @@ def detect_feature_overlaps(genome):
     for sorf in genome['features'].get(bc.FEATURE_SORF, []):
         sorfs = contig_sorfs[sorf['contig']]
         sorfs.append(sorf)
-    
+
     for contig in genome['contigs']:  # find feature overlaps contig-wise to increase the performance
         log.debug('filter features on contig: %s', contig['id'])
 
@@ -119,7 +158,7 @@ def detect_feature_overlaps(genome):
                         "overlap: tRNA (%s) [%i, %i] overlapping with tmRNA (%s) [%i, %i] at %s on contig=%s",
                         tRNA['product'], tRNA['start'], tRNA['stop'], tmRNA['product'], tmRNA['start'], tmRNA['stop'], overlap, tRNA['contig']
                     )
-        
+
         # mark ncRNA-regions overlapping with ncRNA-regions
         for ncRNA_region in contig_ncrna_regions[contig['id']]:
             for ncRNA_region_overlap in contig_ncrna_regions[contig['id']]:
@@ -139,7 +178,7 @@ def detect_feature_overlaps(genome):
                             "overlap: ncRNA-region (%s) [%i, %i] overlapping with ncRNA-region (%s) [%i, %i] at %s on contig=%s, lower bitscore (%f/%f)",
                             ncRNA_region['product'], ncRNA_region['start'], ncRNA_region['stop'], ncRNA_region_overlap['product'], ncRNA_region_overlap['start'], ncRNA_region_overlap['stop'], overlap, ncRNA_region['contig'], ncRNA_region['score'], ncRNA_region_overlap['score']
                         )
-        
+
         # mark CDS overlapping with tRNAs, tmRNAs, rRNAs, CRISPRs
         for cds in contig_cdss[contig['id']]:
             # tmRNA overlaps
@@ -202,7 +241,7 @@ def detect_feature_overlaps(genome):
                         "overlap: CDS (%s/%s) [%i, %i] overlapping CRISPR [%i, %i], %s, contig=%s",
                         cds.get('gene', '-'), cds.get('product', '-'), cds['start'], cds['stop'], crispr['start'], crispr['stop'], overlap, cds['contig']
                     )
-        
+
         # remove sORF overlapping with tRNAs, tmRNAs, rRNAs, CRISPRs, inframe CDSs, shorter inframe sORFs
         for sorf in contig_sorfs[contig['id']]:
             # tmRNA overlaps
@@ -266,7 +305,7 @@ def detect_feature_overlaps(genome):
                         sorf.get('gene', '-'), sorf.get('product', '-'), sorf['start'], sorf['stop'], crispr['start'], crispr['stop'], overlap, sorf['contig']
                     )
             # CDS overlaps (skipped as most overlaps are filtered in sORF detection)
-            
+
             # sORF overlaps
             for overlap_sorf in contig_sorfs[contig['id']]:
                 if(sorf['stop'] < overlap_sorf['start'] or sorf['start'] > overlap_sorf['stop']):
@@ -288,16 +327,16 @@ def detect_feature_overlaps(genome):
                             "overlap: sORF (%s/%s) [%i, %i] overlapping sORF (%s/%s) [%i, %i], %s, contig=%s, lower annotation score (%i/%i)",
                             sorf.get('gene', '-'), sorf.get('product', '-'), sorf['start'], sorf['stop'], overlap_sorf.get('gene', '-'), overlap_sorf.get('product', '-'), overlap_sorf['start'], overlap_sorf['stop'], overlap, sorf['contig'], score_sorf, score_overlap_sorf
                         )
-                    elif(score_sorf == score_overlap_sorf and len(sorf['sequence']) < len(overlap_sorf['sequence'])):  # equal annotation score but shorter sequence -> potential fragment or too short ORF prediction
+                    elif(score_sorf == score_overlap_sorf and len(sorf['aa']) < len(overlap_sorf['aa'])):  # equal annotation score but shorter sequence -> potential fragment or too short ORF prediction
                         overlap = f"[{max(sorf['start'], overlap_sorf['start'])},{min(sorf['stop'], overlap_sorf['stop'])}]"
                         sorf['discarded'] = {
                             'type': bc.DISCARD_TYPE_OVERLAP,
                             'feature_type': bc.FEATURE_SORF,
-                            'description': f"overlaps {bc.FEATURE_SORF} ({overlap_sorf.get('gene', '-')}/{overlap_sorf.get('product', '-')}) at {overlap} with equal score ({score_sorf}) but lower length ({len(sorf['sequence'])}/{len(overlap_sorf['sequence'])})"
+                            'description': f"overlaps {bc.FEATURE_SORF} ({overlap_sorf.get('gene', '-')}/{overlap_sorf.get('product', '-')}) at {overlap} with equal score ({score_sorf}) but lower length ({len(sorf['aa'])}/{len(overlap_sorf['aa'])})"
                         }
                         log.info(
                             "overlap: sORF (%s/%s) [%i, %i] overlapping sORF (%s/%s) [%i, %i], %s, contig=%s, equal annotation score (%i), lower length (%i/%i)",
-                            sorf.get('gene', '-'), sorf.get('product', '-'), sorf['start'], sorf['stop'], overlap_sorf.get('gene', '-'), overlap_sorf.get('product', '-'), overlap_sorf['start'], overlap_sorf['stop'], overlap, sorf['contig'], score_sorf, len(sorf['sequence']), len(overlap_sorf['sequence'])
+                            sorf.get('gene', '-'), sorf.get('product', '-'), sorf['start'], sorf['stop'], overlap_sorf.get('gene', '-'), overlap_sorf.get('product', '-'), overlap_sorf['start'], overlap_sorf['stop'], overlap, sorf['contig'], score_sorf, len(sorf['aa']), len(overlap_sorf['aa'])
                         )
 
 
@@ -307,7 +346,7 @@ def calc_sorf_annotation_score(sorf):
 
     if('ups' in sorf):
         score += 1
-    
+
     ips = sorf.get('ips', None)
     if(ips):
         score += 1
@@ -317,7 +356,7 @@ def calc_sorf_annotation_score(sorf):
         ips_product = ips.get('product', None)
         if(ips_product):
             score += 1
-    
+
     psc = sorf.get('psc', None)
     if(psc):
         score += 1
@@ -332,3 +371,144 @@ def calc_sorf_annotation_score(sorf):
         sorf['contig'], sorf['start'], sorf['stop'], sorf.get('gene', '-'), sorf.get('product', '-'), score
     )
     return score
+
+
+def revise_cds_gene_symbol(feature):
+    raw_gene = feature.get('gene', '')
+
+    raw_genes = raw_gene.replace('/', ',').split(',')  # replace slashes and split raw genes
+
+    revised_genes = []
+    for gene in raw_genes:
+        old_gene = gene
+        if(RE_GENE_SUSPECT_CHARS.search(gene)):  # check for suspect characters -> remove gene symbol
+            log.info('fix gene: remove gene symbol containing suspect chars. old=%s', old_gene)
+            continue
+
+        old_gene = gene
+        gene = gene.replace('gene', '')
+        if(gene != old_gene):  # remove gene literal
+            log.info('fix gene: remove gene literal. new=%s, old=%s', gene, old_gene)
+
+        old_gene = gene
+        if(gene[-1] == '-'):  # remove orphan hyphen
+            gene = gene[:-1]
+            log.info('fix gene: remove orphan hypen. new=%s, old=%s', gene, old_gene)
+        
+        old_gene = gene
+        gene = RE_MULTIWHITESPACE.sub(' ', gene).strip()  # revise whitespaces
+        if(gene != old_gene):
+            log.info('fix gene: revise whitespaces. new=%s, old=%s', gene, old_gene)
+
+        old_gene = gene
+        if(RE_GENE_CAPITALIZED.fullmatch(gene)):
+            gene = gene[0].lower() + gene[1:]
+            log.info('fix gene: lowercase first char. new=%s, old=%s', gene, old_gene)
+
+        if(len(gene) >= 3):
+            if(len(gene) <= 12):
+                revised_genes.append(gene)
+            else:
+                old_gene = gene
+                for part in gene.split(' '):  # try to extract valid gene symbols
+                    m = RE_GENE_SYMBOL.fullmatch(part)
+                    if(m):
+                        symbol = m[0]
+                        revised_genes.append(symbol)
+                        log.info('fix gene: extract symbol from protein name. new=%s, old=%s', symbol, old_gene)
+                    else:
+                        m = RE_PROTEIN_SYMBOL.fullmatch(part)  # extract protein names
+                        if(m):
+                            symbol = m[0]
+                            symbol = symbol[0].lower() + symbol[1:]
+                            revised_genes.append()
+                            log.info('fix gene: extract symbol from protein name. new=%s, old=%s', symbol, old_gene)
+
+    if(len(revised_genes) == 0):
+        feature['gene'] =  None
+    elif(len(revised_genes) == 1):
+        feature['gene'] = revised_genes[0]
+    else:
+        feature['gene'] = ','.join(revised_genes)
+
+
+def revise_cds_product(feature):
+    """Revise product name for INSDC compliant submissions"""
+    product = feature['product']
+    
+    old_product = product
+    product = RE_PROTEIN_WEIGHT.sub(' ', product)  # remove protein weight in (k)Da
+    if(product != old_product):
+        log.info('fix product: remove protein weight in (k)Da. new=%s, old=%s', product, old_product)
+
+    old_product = product
+    product = re.sub(RE_PROTEIN_PERIOD_SEPARATOR, r'\1-\2', product)  # replace separator periods
+    if(product != old_product):
+        log.info('fix product: replace separator periods. new=%s, old=%s', product, old_product)
+
+    old_product = product
+    product = RE_PROTEIN_SUSPECT_CHARS.sub('', product)  # remove suspect characters
+    if(product != old_product):
+        log.info('fix product: replace invalid characters (.@=?%%). new=%s, old=%s', product, old_product)
+
+    old_product = product
+    product = product.replace('FOG:', '')  # remove FOG ids
+    if(product != old_product):
+        log.info('fix product: replace FOG ids. new=%s, old=%s', product, old_product)
+
+    old_product = product
+    dufs = []  # replace DUF-containing products
+    for m in RE_DOMAIN_OF_UNKNOWN_FUCTION.finditer(product):
+        dufs.append(m.group(1).upper())
+    if(len(dufs) >= 1):
+        product = f"{' '.join(dufs)} domain{'s' if len(dufs) > 1 else ''}-containing protein"
+        if(product != old_product):
+            log.info('fix product: revise DUF. new=%s, old=%s', product, old_product)
+    
+    old_product = product
+    if('conserved' in product.lower()):  # replace conserved UPF proteins
+        upfs = []
+        for m in RE_UNCHARACTERIZED_PROTEIN_FAMILY.finditer(product):
+            upfs.append(m.group(1).upper())
+        if(len(upfs) >= 1):
+            product = f"{' '.join(upfs)} protein"
+            if(product != old_product):
+                log.info('fix product: revise UPF. new=%s, old=%s', product, old_product)
+
+    old_product = product
+    product = RE_PROTEIN_HOMOLOG.sub('-like protein', product)  # replace Homologs
+    if(product != old_product):
+        if(product.count('protein') == 2):
+            product = product.replace('protein', '', 1)  # remove former protein term if existing
+        log.info('fix product: replace Homolog. new=%s, old=%s', product, old_product)
+
+    old_product = product
+    product = RE_MULTIWHITESPACE.sub(' ', product).strip()  # revise whitespaces
+    if(product != old_product):
+        log.info('fix product: revise whitespaces. new=%s, old=%s', product, old_product)
+
+    old_product = product
+    product = RE_PROTEIN_PUTATIVE.sub('putative', product)  # replace putative synonyms)
+    if(product != old_product):
+        log.info('fix product: replace putative synonyms. new=%s, old=%s', product, old_product)
+
+    old_product = product
+    if(RE_PROTEIN_DOMAIN_CONTAINING.search(product)):  # replace domain name underscores in domain names
+        product = product.replace('_', '-')
+        if(product != old_product):
+            log.info('fix product: replace domain name underscores. new=%s, old=%s', product, old_product)
+
+    old_product = product
+
+
+    if(
+        RE_PROTEIN_CONTIG.search(product) or  # protein containing 'contig'
+        RE_PROTEIN_NODE.search(product) or  # potential contig name (SPAdes)
+        RE_PROTEIN_POTENTIAL_CONTIG_NAME.search(product) or  # potential contig name (SPAdes)
+        RE_PROTEIN_NO_LETTERS.fullmatch(product)  # no letters -> set to Hypothetical
+        ):  # remove suspect products and mark as hypothetical
+        product = bc.HYPOTHETICAL_PROTEIN
+        feature['hypothetical'] = True
+        log.info('remove product: mark proteins with suspect products as hypothetical. old=%s', old_product)
+
+    feature['product'] = product
