@@ -1,7 +1,6 @@
-
 import logging
-import re
 import subprocess as sp
+
 from collections import OrderedDict
 
 from Bio import SeqIO
@@ -9,16 +8,17 @@ from Bio.SeqUtils.ProtParam import ProteinAnalysis
 
 import bakta.config as cfg
 import bakta.constants as bc
+import bakta.features.orf as orf
+import bakta.io.fasta as fasta
 import bakta.utils as bu
 import bakta.so as so
-import bakta.io.fasta as fasta
+
 
 log = logging.getLogger('CDS')
 
 
 def predict(genome, sequences_path):
     """Predict open reading frames with Prodigal."""
-
     # create prodigal trainining file if not provided by the user
     prodigal_tf_path = cfg.prodigal_tf
     if(prodigal_tf_path is None):
@@ -47,13 +47,13 @@ def predict(genome, sequences_path):
         cds = parse_prodigal_output(genome, sequences, gff_contigs_path, proteins_contigs_path)
         log.info('contig cds: predicted=%i', len(cds))
         cdss.extend(cds)
-    
+
     # execute prodigal for complete replicons (chromosomes/plasmids)
     replicons_path = cfg.tmp_path.joinpath('replicons.fasta')
     replicons = [c for c in genome['contigs'] if c['complete']]
     if(len(replicons) > 0):
         fasta.export_contigs(replicons, replicons_path)
-        log.debug('export replicons: # contigs=%i, path=%s', len(replicons), replicons_path)
+        log.debug('export replicons: # sequences=%i, path=%s', len(replicons), replicons_path)
         proteins_replicons_path = cfg.tmp_path.joinpath('prodigal.proteins.replicons.faa')
         gff_replicons_path = cfg.tmp_path.joinpath('prodigal.replicons.gff')
         log.info('run prodigal: type=replicons, # sequences=%i', len(replicons))
@@ -61,7 +61,7 @@ def predict(genome, sequences_path):
         cds = parse_prodigal_output(genome, sequences, gff_replicons_path, proteins_replicons_path)
         log.info('replicon cds: predicted=%i', len(cds))
         cdss.extend(cds)
-    
+
     log.info('predicted=%i', len(cdss))
     return cdss
 
@@ -111,8 +111,6 @@ def execute_prodigal(genome, contigs_path, traininng_file_path, proteins_path=No
 
 def parse_prodigal_output(genome, sequences, gff_path, proteins_path):
     log.debug('parse-prodigal-output: gff-path=%s, proteins-path=%s', gff_path, proteins_path)
-    # parse orfs
-    # TODO: replace code by BioPython GFF3 parser
     cdss = {}
     partial_cdss_per_record = {}
     partial_cdss_per_contig = {k['id']: [] for k in genome['contigs']}
@@ -132,14 +130,14 @@ def parse_prodigal_output(genome, sequences, gff_path, proteins_path):
                 cds['gene'] = None
                 cds['product'] = None
                 cds['start_type'] = gff_annotations['start_type']
-                cds['rbs_motif'] = gff_annotations['rbs_motif']
+                cds['rbs_motif'] = gff_annotations['rbs_motif'] if gff_annotations['rbs_motif'] != 'None' else None
                 cds['db_xrefs'] = [so.SO_CDS.id]
-                
+
                 if(cds['strand'] == bc.STRAND_FORWARD):
                     cds['frame'] = (cds['start'] - 1) % 3 + 1
                 else:
                     cds['frame'] = (sequences[cds['contig']]['length'] - cds['stop']) % 3 + 1
-                
+
                 if(gff_annotations['partial'] == '10'):
                     cds['truncated'] = bc.FEATURE_END_5_PRIME if cds['strand'] == bc.STRAND_FORWARD else bc.FEATURE_END_3_PRIME
                     partial_cdss_per_record[f"{cds['contig']}_{contig_orf_id}"] = cds
@@ -150,7 +148,7 @@ def parse_prodigal_output(genome, sequences, gff_path, proteins_path):
                     partial_cdss_per_contig[cds['contig']].append(cds)
                 else:
                     cdss[f"{cds['contig']}_{contig_orf_id}"] = cds
-                
+
                 log.info(
                     'contig=%s, start=%i, stop=%i, strand=%s, frame=%s, truncated=%s, start-type=%s, RBS-motif=%s',
                     cds['contig'], cds['start'], cds['stop'], cds['strand'], cds['frame'], cds.get('truncated', 'no'), cds['start_type'], cds['rbs_motif']
@@ -161,17 +159,17 @@ def parse_prodigal_output(genome, sequences, gff_path, proteins_path):
         for record in SeqIO.parse(fh, 'fasta'):
             cds = cdss.get(record.id, None)
             if(cds):
-                seq = str(record.seq)[:-1]  # discard trailing asterisk
-                cds['sequence'] = seq
-                cds['aa_digest'], cds['aa_hexdigest'] = bu.calc_aa_hash(seq)
+                aa = str(record.seq)[:-1]  # discard trailing asterisk
+                cds['aa'] = aa
+                cds['aa_digest'], cds['aa_hexdigest'] = bu.calc_aa_hash(aa)
             else:
                 partial_cds = partial_cdss_per_record.get(record.id, None)
                 if(partial_cds):
-                    seq = str(record.seq)
+                    aa = str(record.seq)
                     if(partial_cds['truncated'] == bc.FEATURE_END_5_PRIME):
-                        seq = seq[:-1]  # discard trailing asterisk
-                    partial_cds['sequence'] = seq
-                    partial_cds['aa_digest'], partial_cds['aa_hexdigest'] = bu.calc_aa_hash(seq)
+                        aa = aa[:-1]  # discard trailing asterisk
+                    partial_cds['aa'] = aa
+                    partial_cds['aa_digest'], partial_cds['aa_hexdigest'] = bu.calc_aa_hash(aa)
                 else:
                     log.warning('unknown sequence detected! id=%s, sequence=%s', record.id, record.seq)
     cdss = list(cdss.values())
@@ -181,10 +179,8 @@ def parse_prodigal_output(genome, sequences, gff_path, proteins_path):
         if(len(partial_cdss) >= 2):  # skip unpaired edge features
             first_partial_cds = partial_cdss[0]  # first partial CDS per contig
             last_partial_cds = partial_cdss[-1]  # last partial CDS per contig
-            # check if partial CDS are on same strand
-            # and have opposite truncated edges
-            # and firtst starts at 1
-            # and last ends at contig end (length)
+            # check if partial CDSs are on same strand and have opposite truncated edges
+            # and firtst starts at 1 and last ends at contig end (length)
             if(first_partial_cds['strand'] == last_partial_cds['strand']
                 and first_partial_cds['truncated'] != last_partial_cds['truncated']
                 and first_partial_cds['start'] == 1
@@ -193,30 +189,40 @@ def parse_prodigal_output(genome, sequences, gff_path, proteins_path):
                 cds = last_partial_cds
                 cds['stop'] = first_partial_cds['stop']
                 if(last_partial_cds['truncated'] == bc.FEATURE_END_3_PRIME):
-                    seq = last_partial_cds['sequence'] + first_partial_cds['sequence']  # merge sequence
+                    aa = last_partial_cds['aa'] + first_partial_cds['aa']  # merge sequence
                 else:
-                    seq = first_partial_cds['sequence'] + last_partial_cds['sequence']  # merge sequence
+                    aa = first_partial_cds['aa'] + last_partial_cds['aa']  # merge sequence
                     cds['start_type'] = first_partial_cds['start_type']
                     cds['rbs_motif'] = first_partial_cds['rbs_motif']
-                log.debug(f'trunc seq: seq-start={seq[:10]}, seq-end={seq[-10:]}')
+                log.debug(f'trunc seq: seq-start={aa[:10]}, seq-end={aa[-10:]}')
 
                 cds['edge'] = True  # mark CDS as edge feature
                 cds.pop('truncated')
-                
-                cds['sequence'] = seq
-                cds['aa_digest'], cds['aa_hexdigest'] = bu.calc_aa_hash(seq)
+
+                cds['aa'] = aa
+                cds['aa_digest'], cds['aa_hexdigest'] = bu.calc_aa_hash(aa)
                 cdss.append(cds)
                 log.info(
-                    'edge CDS: contig=%s, start=%i, stop=%i, strand=%s, frame=%s, start-type=%s, RBS-motif=%s, aa-hexdigest=%s, seq=[%s..%s]',
-                    cds['contig'], cds['start'], cds['stop'], cds['strand'], cds['frame'], cds['start_type'], cds['rbs_motif'], cds['aa_hexdigest'], seq[:10], seq[-10:]
+                    'edge CDS: contig=%s, start=%i, stop=%i, strand=%s, frame=%s, start-type=%s, RBS-motif=%s, aa-hexdigest=%s, aa=[%s..%s]',
+                    cds['contig'], cds['start'], cds['stop'], cds['strand'], cds['frame'], cds['start_type'], cds['rbs_motif'], cds['aa_hexdigest'], aa[:10], aa[-10:]
                 )
                 partial_cdss = partial_cdss[1:-1]
         for partial_cds in partial_cdss:
             cdss.append(partial_cds)
             log.info(
-                'truncated CDS: contig=%s, start=%i, stop=%i, strand=%s, frame=%s, truncated=%s, start-type=%s, RBS-motif=%s, aa-hexdigest=%s, seq=[%s..%s]',
-                partial_cds['contig'], partial_cds['start'], partial_cds['stop'], partial_cds['strand'], partial_cds['frame'], partial_cds['truncated'], partial_cds['start_type'], partial_cds['rbs_motif'], partial_cds['aa_hexdigest'], partial_cds['sequence'][:10], partial_cds['sequence'][-10:]
+                'truncated CDS: contig=%s, start=%i, stop=%i, strand=%s, frame=%s, truncated=%s, start-type=%s, RBS-motif=%s, aa-hexdigest=%s, aa=[%s..%s]',
+                partial_cds['contig'], partial_cds['start'], partial_cds['stop'], partial_cds['strand'], partial_cds['frame'], partial_cds['truncated'], partial_cds['start_type'], partial_cds['rbs_motif'], partial_cds['aa_hexdigest'], partial_cds['aa'][:10], partial_cds['aa'][-10:]
             )
+
+    contigs = {c['id']: c for c in genome['contigs']}
+    for cds in cdss:  # extract nt sequences
+        contig = contigs[cds['contig']]
+        nt = bu.extract_feature_sequence(cds, contig)
+        cds['nt'] = nt
+        log.info(
+            'contig=%s, start=%i, stop=%i, strand=%s, nt=[%s..%s]',
+            cds['contig'], cds['start'], cds['stop'], cds['strand'], nt[:10], nt[-10:]
+        )
     return cdss
 
 
@@ -235,16 +241,13 @@ def split_gff_annotation(annotation_string):
 def predict_pfam(cdss):
     """Detect Pfam-A entries"""
     fasta_path = cfg.tmp_path.joinpath('hypotheticals.faa')
-    with fasta_path.open(mode='w') as fh:
-        for cds in cdss:
-            fh.write(f">{cds['aa_hexdigest']}\n{cds['sequence']}\n")
-    
-    output_path = cfg.tmp_path.joinpath('cds.pfam.hmm.tsv')
+    orf.write_internal_faa(cdss, fasta_path)
+    output_path = cfg.tmp_path.joinpath('cds.hypotheticals.pfam.hmm.tsv')
     cmd = [
         'hmmsearch',
         '--noali',
         '--cut_ga',  # use gathering cutoff
-        '--tblout', str(output_path),
+        '--domtblout', str(output_path),
         '--cpu', str(cfg.threads if cfg.threads <= 4 else 4),
         str(cfg.db_path.joinpath('pfam')),
         str(fasta_path)
@@ -265,20 +268,33 @@ def predict_pfam(cdss):
 
     pfam_hits = []
     cds_pfams_hits = {}
-    orf_by_aa_digest = {cds['aa_hexdigest']: cds for cds in cdss}
+    orf_by_aa_digest = orf.get_orf_dictionary(cdss)
     with output_path.open() as fh:
         for line in fh:
             if(line[0] != '#'):
-                cols = re.split('\s+', line.strip())
-                aa_hexdigest = cols[0]
-                cds = orf_by_aa_digest[aa_hexdigest]
-                
+                cols = bc.RE_MULTIWHITESPACE.split(line.strip())
+                aa_identifier = cols[0]
+                cds = orf_by_aa_digest[aa_identifier]
+
+                domain_length = int(cols[5])
+                domain_start = int(cols[15])
+                domain_stop = int(cols[16])
+                domain_cov = (domain_stop - domain_start + 1) / domain_length
+                aa_start = int(cols[19])
+                aa_stop = int(cols[20])
+                aa_cov = (aa_stop - aa_start + 1) / len(cds['aa'])
+
                 pfam = OrderedDict()
-                pfam['id'] = cols[3]
-                pfam['name'] = cols[2]
-                pfam['evalue'] = float(cols[4])
-                pfam['score'] = float(cols[5])
-                
+                pfam['id'] = cols[4]
+                pfam['name'] = cols[3]
+                pfam['length'] = domain_length
+                pfam['aa_cov'] = aa_cov
+                pfam['hmm_cov'] = domain_cov
+                pfam['evalue'] = float(cols[6])
+                pfam['score'] = float(cols[7])
+                pfam['start'] = aa_start
+                pfam['stop'] = aa_stop
+
                 if('pfams' not in cds):
                     cds['pfams'] = []
                 cds['pfams'].append(pfam)
@@ -286,10 +302,10 @@ def predict_pfam(cdss):
                     cds['dbxrefs'] = []
                 cds['dbxrefs'].append(f"PFAM:{pfam['id']}")
                 pfam_hits.append(cds)
-                cds_pfams_hits[aa_hexdigest] = cds
+                cds_pfams_hits[aa_identifier] = cds
                 log.info(
-                    'pfam detected: contig=%s, start=%i, stop=%i, strand=%s, pfam-id=%s, evalue=%1.1e, bitscore=%f, name=%s',
-                    cds['contig'], cds['start'], cds['stop'], cds['strand'], pfam['id'], pfam['evalue'], pfam['score'], pfam['name']
+                    'pfam detected: contig=%s, start=%i, stop=%i, strand=%s, pfam-id=%s, length=%i, aa-start=%i, aa-stop=%i, aa-cov=%1.1f, hmm-cov=%1.1f, evalue=%1.1e, bitscore=%1.1f, name=%s',
+                    cds['contig'], cds['start'], cds['stop'], cds['strand'], pfam['id'], pfam['length'], pfam['start'], pfam['stop'], pfam['aa_cov'], pfam['hmm_cov'], pfam['evalue'], pfam['score'], pfam['name']
                 )
     log.info('predicted-pfams=%i, CDS-w/-pfams=%i', len(pfam_hits), len(cds_pfams_hits))
     return cds_pfams_hits.values()
@@ -297,7 +313,7 @@ def predict_pfam(cdss):
 
 def analyze_proteins(cdss):
     for cds in cdss:
-        seq = ProteinAnalysis(cds['sequence'])
+        seq = ProteinAnalysis(cds['aa'])
         seq_stats = OrderedDict()
         try:
             seq_stats['molecular_weight'] = seq.molecular_weight()
@@ -315,4 +331,52 @@ def analyze_proteins(cdss):
                 cds['contig'], cds['start'], cds['stop'], cds['strand'], cds['frame']
             )
             seq_stats['isoelectric_point'] = None
-        cds['seq_stats'] = seq_stats
\ No newline at end of file
+        cds['seq_stats'] = seq_stats
+
+
+def revise_special_cases(cdss):
+    """
+    Revise rare but known special cases as for istance supposedly truncated dnaA genes on rotated chromosome starts
+    which often appear on re-annotated genomes.
+    """
+    
+    # look for supposedly truncated dnaA genes on rotated chromosome starts: start=1, strand=+
+    dnaA = None
+    for cds in cdss:
+        if(
+            cds['start'] == 1 and 
+            cds['strand'] == bc.STRAND_FORWARD and 
+            cds['start_type'] == 'Edge' and 
+            cds['rbs_motif'] is None and
+            ('dnaa' in cds['product'].lower().split() or cds['gene'] == 'dnaA')):
+            dnaA = cds
+            break
+    if(dnaA is not None):
+        dnaA.pop('truncated')
+        gene = dnaA.get('gene', None)
+        gene = gene if gene is not None else '-'
+        log.info(
+            'revise supposedly truncated dnaA gene on rotated chromosome start: contig=%s, start=%i, stop=%i, strand=%s, gene=%s, product=%s, nt=[%s..%s], aa=[%s..%s]',
+            dnaA['contig'], dnaA['start'], dnaA['stop'], dnaA['strand'], gene, dnaA['product'], dnaA['nt'][:10], dnaA['nt'][-10:], dnaA['aa'][:10], dnaA['aa'][-10:]
+        )
+    
+    # look for supposedly truncated repA genes on rotated plasmid starts: start=1, strand=+
+    repA = None
+    for cds in cdss:
+        if(
+            cds['start'] == 1 and 
+            cds['strand'] == bc.STRAND_FORWARD and 
+            cds['start_type'] == 'Edge' and 
+            cds['rbs_motif'] is None and
+            ('repa' in cds['product'].lower().split() or cds['gene'] == 'repA')):
+            repA = cds
+            break
+    if(repA is not None):
+        repA.pop('truncated')
+        gene = repA.get('gene', None)
+        gene = gene if gene is not None else '-'
+        log.info(
+            'revise supposedly truncated repA gene on rotated plasmid start: contig=%s, start=%i, stop=%i, strand=%s, gene=%s, product=%s, nt=[%s..%s], aa=[%s..%s]',
+            repA['contig'], repA['start'], repA['stop'], repA['strand'], gene, repA['product'], repA['nt'][:10], repA['nt'][-10:], repA['aa'][:10], repA['aa'][-10:]
+        )
+    
\ No newline at end of file
