@@ -1,12 +1,13 @@
-
 import logging
-import re
 import subprocess as sp
+
 from collections import OrderedDict
 
 import bakta.config as cfg
 import bakta.constants as bc
 import bakta.so as so
+import bakta.utils as bu
+
 
 log = logging.getLogger('NC_RNA')
 
@@ -55,13 +56,16 @@ def predict_nc_rnas(genome, contigs_path):
                 rfam2go[rfam] = [go]
 
     ncrnas = []
+    contigs = {c['id']: c for c in genome['contigs']}
     with output_path.open() as fh:
         for line in fh:
             if(line[0] != '#'):
-                (subject, accession, contig_id, contig_acc, mdl, mdl_from, mdl_to,
+                (
+                    subject, accession, contig_id, contig_acc, mdl, mdl_from, mdl_to,
                     start, stop, strand, trunc, passed, gc, bias, score, evalue,
-                    inc, description) = re.split('\s+', line.strip(), maxsplit=17)
-                
+                    inc, description
+                ) = bc.RE_MULTIWHITESPACE.split(line.strip(), maxsplit=17)
+
                 if(strand == '-'):
                     (start, stop) = (stop, start)
                 (start, stop) = (int(start), int(stop))
@@ -74,18 +78,18 @@ def predict_nc_rnas(genome, contigs_path):
                     truncated = bc.FEATURE_END_3_PRIME
                 else:
                     truncated = None
-                
+
                 if(evalue > 1E-4):
                     log.debug(
                         'discard low E value: contig=%s, start=%i, stop=%i, strand=%s, gene=%s, length=%i, truncated=%s, score=%1.1f, evalue=%1.1e',
                         contig_id, start, stop, strand, subject, length, truncated, score, evalue
                     )
                 else:
-                    rfam_id = f'RFAM:{accession}'
+                    rfam_id = f'{bc.DB_XREF_RFAM}:{accession}'
                     db_xrefs = [rfam_id]
                     if(rfam_id in rfam2go):
                         db_xrefs += rfam2go[rfam_id]
-                    
+
                     ncrna = OrderedDict()
                     ncrna['type'] = bc.FEATURE_NC_RNA
                     ncrna['class'] = determine_class(description)
@@ -103,23 +107,26 @@ def predict_nc_rnas(genome, contigs_path):
                         ncrna['product'] = f"(5' truncated) {description}"
                     elif(truncated == bc.FEATURE_END_3_PRIME):
                         ncrna['product'] = f"(3' truncated) {description}"
-                    
+
                     if(ncrna['class'] is not None):
                         db_xrefs.append(ncrna['class'].id)
                     else:
                         db_xrefs.append(so.SO_NCRNA_GENE.id)
-                    
+
                     if(truncated):
                         ncrna['truncated'] = truncated
-                    
+
                     ncrna['score'] = score
                     ncrna['evalue'] = evalue
                     ncrna['db_xrefs'] = db_xrefs
 
+                    nt = bu.extract_feature_sequence(ncrna, contigs[contig_id])  # extract nt sequences
+                    ncrna['nt'] = nt
+
                     ncrnas.append(ncrna)
                     log.info(
-                        'contig=%s, start=%i, stop=%i, strand=%s, gene=%s, product=%s, length=%i, truncated=%s, score=%1.1f, evalue=%1.1e',
-                        ncrna['contig'], ncrna['start'], ncrna['stop'], ncrna['strand'], ncrna['gene'], ncrna['product'], length, truncated, ncrna['score'], ncrna['evalue']
+                        'contig=%s, start=%i, stop=%i, strand=%s, gene=%s, product=%s, length=%i, truncated=%s, score=%1.1f, evalue=%1.1e, nt=[%s..%s]',
+                        ncrna['contig'], ncrna['start'], ncrna['stop'], ncrna['strand'], ncrna['gene'], ncrna['product'], length, truncated, ncrna['score'], ncrna['evalue'], nt[:10], nt[-10:]
                     )
     log.info('predicted=%i', len(ncrnas))
     return ncrnas
@@ -134,4 +141,4 @@ def determine_class(description):
     elif('antisense' in description):
         return so.SO_NCRNA_GENE_ANTISENSE
     else:
-        None
\ No newline at end of file
+        None
