@@ -1,12 +1,13 @@
-
 import logging
-import re
 import subprocess as sp
+
 from collections import OrderedDict
 
 import bakta.config as cfg
 import bakta.constants as bc
 import bakta.so as so
+import bakta.utils as bu
+
 
 log = logging.getLogger('NC_RNA_REGION')
 
@@ -55,13 +56,14 @@ def predict_nc_rna_regions(genome, contigs_path):
                 rfam2go[rfam] = [go]
 
     ncrnas = []
+    contigs = {c['id']: c for c in genome['contigs']}
     with output_path.open() as fh:
         for line in fh:
             if(line[0] != '#'):
                 (subject, accession, contig_id, contig_acc, mdl, mdl_from, mdl_to,
                     start, stop, strand, trunc, passed, gc, bias, score, evalue,
-                    inc, description) = re.split('\s+', line.strip(), maxsplit=17)
-                
+                    inc, description) = bc.RE_MULTIWHITESPACE.split(line.strip(), maxsplit=17)
+
                 if(strand == '-'):
                     (start, stop) = (stop, start)
                 (start, stop) = (int(start), int(stop))
@@ -74,18 +76,18 @@ def predict_nc_rna_regions(genome, contigs_path):
                     truncated = bc.FEATURE_END_3_PRIME
                 else:
                     truncated = None
-                
+
                 if(evalue > 1E-4):
                     log.debug(
                         'discard low E value: contig=%s, start=%i, stop=%i, strand=%s, gene=%s, length=%i, truncated=%s, score=%1.1f, evalue=%1.1e',
                         contig_id, start, stop, strand, subject, length, truncated, score, evalue
                     )
                 else:
-                    rfam_id = f'RFAM:{accession}'
+                    rfam_id = f'{bc.DB_XREF_RFAM}:{accession}'
                     db_xrefs = [rfam_id]
                     if(rfam_id in rfam2go):
                         db_xrefs += rfam2go[rfam_id]
-                    
+
                     ncrna_region = OrderedDict()
                     ncrna_region['type'] = bc.FEATURE_NC_RNA_REGION
                     ncrna_region['class'] = determine_class(description)
@@ -103,19 +105,22 @@ def predict_nc_rna_regions(genome, contigs_path):
                         ncrna_region['product'] = f"(5' truncated) {description}"
                     elif(truncated == bc.FEATURE_END_3_PRIME):
                         ncrna_region['product'] = f"(3' truncated) {description}"
-                    
+
                     if(ncrna_region['class'] is not None):
                         db_xrefs.append(ncrna_region['class'].id)
                     else:
                         db_xrefs.append(so.SO_REGULATORY_REGION.id)
-                    
+
                     if(truncated):
                         ncrna_region['truncated'] = truncated
-                    
+
                     ncrna_region['score'] = score
                     ncrna_region['evalue'] = evalue
                     ncrna_region['db_xrefs'] = db_xrefs
 
+                    nt = bu.extract_feature_sequence(ncrna_region, contigs[contig_id])  # extract nt sequences
+                    ncrna_region['nt'] = nt
+
                     ncrnas.append(ncrna_region)
                     log.info(
                         'contig=%s, start=%i, stop=%i, strand=%s, label=%s, product=%s, length=%i, truncated=%s, score=%1.1f, evalue=%1.1e',
