@@ -1,24 +1,25 @@
-
+import concurrent.futures as cf
 import logging
 import math
 import subprocess as sp
+
 from collections import OrderedDict
-import concurrent.futures as cf
 
 from Bio.Seq import Seq
 
 import bakta.config as cfg
 import bakta.constants as bc
+import bakta.features.orf as orf
 import bakta.utils as bu
 import bakta.psc as psc
 import bakta.so as so
 
+
 log = logging.getLogger('S_ORF')
 
 
 def extract(genome):
     """Predict open reading frames in mem via BioPython."""
-
     orfs = []
     for contig in genome['contigs']:
         dna_seq = Seq(contig['sequence'])
@@ -37,15 +38,15 @@ def extract(genome):
                 while aa_start > -1 and aa_end > -1:
                     orf_length = aa_end - aa_start
                     if(orf_length >= bc.MIN_SORF_LENGTH and orf_length < bc.MAX_SORF_LENGTH):  # get all CDS starts (M)
+                        aa = aa_seq[aa_start:aa_end]
+                        (aa_digest, aa_hexdigest) = bu.calc_aa_hash(aa)
                         if(strand == bc.STRAND_FORWARD):
                             dna_start = aa_start * 3 + frame + 1  # +1: 0 based idx to 1 based
                             dna_stop = aa_end * 3 + 2 + frame + 1
                         else:
                             dna_start = len(seq) - frame - (aa_end + 1) * 3 + 1
                             dna_stop = len(seq) - frame - aa_start * 3
-                        sequence = aa_seq[aa_start:aa_end]
-                        (aa_digest, aa_hexdigest) = bu.calc_aa_hash(sequence)
-
+                        
                         sorf = OrderedDict()
                         sorf['type'] = bc.FEATURE_SORF
                         sorf['contig'] = contig['id']
@@ -56,14 +57,17 @@ def extract(genome):
                         sorf['product'] = None
                         sorf['frame'] = frame + 1
                         sorf['db_xrefs'] = [so.SO_SORF.id]
-                        sorf['sequence'] = sequence
+                        sorf['aa'] = aa
                         sorf['aa_digest'] = aa_digest
                         sorf['aa_hexdigest'] = aa_hexdigest
                         
+                        nt = bu.extract_feature_sequence(sorf, contig)  # extract nt sequences
+                        sorf['nt'] = nt
+
                         orfs.append(sorf)
                         log.debug(
-                            'contig=%s, start=%i, stop=%i, strand=%s, frame=%i, length=%i, seq=%s',
-                            contig['id'], sorf['start'], sorf['stop'], strand, frame, len(sequence), sequence
+                            'contig=%s, start=%i, stop=%i, strand=%s, frame=%i, aa-length=%i, aa=%s, nt=[%s..%s]',
+                            contig['id'], sorf['start'], sorf['stop'], strand, frame, len(aa), aa, nt[:10], nt[-10:]
                         )
                     aa_start = aa_seq.find('M', aa_start + 1)
                     if(aa_start > aa_end):
@@ -74,16 +78,15 @@ def extract(genome):
 
 
 def get_feature_start(feature):
-    return feature['start'] if feature['strand'] == '+' else feature['stop']
+    return feature['start'] if feature['strand'] == bc.STRAND_FORWARD else feature['stop']
 
 
 def get_feature_stop(feature):
-    return feature['stop'] if feature['strand'] == '+' else feature['start']
+    return feature['stop'] if feature['strand'] == bc.STRAND_FORWARD else feature['start']
 
 
 def overlap_filter(genome, orfs_raw):
     """Filter in-mem ORFs by overlapping CDSs."""
-
     t_rnas_per_contig = {k['id']: [] for k in genome['contigs']}
     for t_rna in genome['features'].get(bc.FEATURE_T_RNA, []):
         t_rnas = t_rnas_per_contig[t_rna['contig']]
@@ -121,7 +124,7 @@ def overlap_filter(genome, orfs_raw):
         orfs.append(sorf)
 
     discarded_sorf_keys = set()
-    with cf.ProcessPoolExecutor(max_workers=cfg.threads - 1) as tpe:
+    with cf.ProcessPoolExecutor(max_workers=cfg.threads) as tpe:
         futures = []
         for contig in genome['contigs']:
             contig_sorfs = sorfs_per_contig[contig['id']]
@@ -142,12 +145,12 @@ def overlap_filter(genome, orfs_raw):
         for f in futures:
             for sorf_key in [sk for sk in f.result() if sk is not None]:
                 discarded_sorf_keys.add(sorf_key)
-    
+
     valid_sorfs = []
     discarded_sorfs = []
     for sorfs in sorfs_per_contig.values():
         for sorf in sorfs:
-            key = f"{sorf['contig']}-{sorf['start']}-{sorf['stop']}-{sorf['strand']}-{sorf['aa_hexdigest']}"
+            key = orf.get_orf_key(sorf)
             if(key in discarded_sorf_keys):
                 discarded_sorfs.append(sorf)
             else:
@@ -160,6 +163,7 @@ def overlap_filter(genome, orfs_raw):
 def filter_sorf(sorf_chunk, contig_cdss, contig_r_rnas, contig_t_rnas, contig_crispr_arrays):
     discarded_sorf_keys = []
     for sorf in sorf_chunk:
+        break_flag = False
         # filter CDS overlapping ORFs
         for cds in contig_cdss:
             # log.debug('filter short ORFs by CDS: %s%i[%i->%i]', cds['strand'], cds['frame'], cds['start'], cds['stop'])
@@ -169,40 +173,48 @@ def filter_sorf(sorf_chunk, contig_cdss, contig_r_rnas, contig_t_rnas, contig_cr
                     if(cds['stop'] < sorf['start'] or cds['start'] > sorf['stop']):
                         continue
                     else:
-                        discarded_sorf_keys.append(f"{sorf['contig']}-{sorf['start']}-{sorf['stop']}-{sorf['strand']}-{sorf['aa_hexdigest']}")
+                        discarded_sorf_keys.append(orf.get_orf_key(sorf))
+                        break_flag = True
+                        break
                 else:
-                    if(sorf['start'] < cds['start'] and sorf['stop'] > cds['start']):
-                        # out-of-frame sorf partially overlapping CDS upstream
-                        # ToDo: add max overlap threshold
-                        continue
-                    elif(sorf['start'] >= cds['start'] and sorf['stop'] <= cds['stop']):
+                    # if(sorf['start'] < cds['start'] and sorf['stop'] > cds['start']):  # ToDo: add max overlap threshold
+                    #     # out-of-frame sorf partially overlapping CDS upstream
+                    #     continue
+                    # elif(sorf['start'] >= cds['start'] and sorf['stop'] <= cds['stop']):
+                    if(sorf['start'] >= cds['start'] and sorf['stop'] <= cds['stop']):
                         # out-of-frame sorf completely overlapped by CDS
-                        discarded_sorf_keys.append(f"{sorf['contig']}-{sorf['start']}-{sorf['stop']}-{sorf['strand']}-{sorf['aa_hexdigest']}")
-                    elif(sorf['start'] < cds['stop'] and sorf['stop'] > cds['stop']):
-                        # out-of-frame sorf partially overlapping CDS downstream
-                        # ToDo: add max overlap threshold
-                        continue
+                        discarded_sorf_keys.append(orf.get_orf_key(sorf))
+                        break_flag = True
+                        break
+                    # elif(sorf['start'] < cds['stop'] and sorf['stop'] > cds['stop']):  # ToDo: add max overlap threshold
+                    #     # out-of-frame sorf partially overlapping CDS downstream
+                    #     continue
             else:
                 if(sorf['frame'] == cds['frame']):
                     # fast/simple overlap detection for in frame orientation
                     if(cds['stop'] < sorf['start'] or cds['start'] > sorf['stop']):
                         continue
                     else:
-                        discarded_sorf_keys.append(f"{sorf['contig']}-{sorf['start']}-{sorf['stop']}-{sorf['strand']}-{sorf['aa_hexdigest']}")
+                        discarded_sorf_keys.append(orf.get_orf_key(sorf))
+                        break_flag = True
+                        break
                 else:
                     # ToDo: add out-of-frame filters
-                    if(sorf['start'] < cds['start'] and sorf['stop'] > cds['start']):
-                        # out-frame sorf partially overlapping CDS upstream
-                        # ToDo: add max overlap threshold
-                        continue
-                    elif(sorf['start'] >= cds['start'] and sorf['stop'] <= cds['stop']):
+                    # if(sorf['start'] < cds['start'] and sorf['stop'] > cds['start']):  # ToDo: add max overlap threshold
+                    #     # out-frame sorf partially overlapping CDS upstream
+                    #     continue
+                    # elif(sorf['start'] >= cds['start'] and sorf['stop'] <= cds['stop']):
+                    if(sorf['start'] >= cds['start'] and sorf['stop'] <= cds['stop']):
                         # out-frame sorf completely overlapped by CDS
-                        discarded_sorf_keys.append(f"{sorf['contig']}-{sorf['start']}-{sorf['stop']}-{sorf['strand']}-{sorf['aa_hexdigest']}")
-                    elif(sorf['start'] < cds['stop'] and sorf['stop'] > cds['stop']):
-                        # out-frame sorf partially overlapping CDS downstream
-                        # ToDo: add max overlap threshold
-                        continue
-                    
+                        discarded_sorf_keys.append(orf.get_orf_key(sorf))
+                        break_flag = True
+                        break
+                    # elif(sorf['start'] < cds['stop'] and sorf['stop'] > cds['stop']):  # ToDo: add max overlap threshold
+                    #     # out-frame sorf partially overlapping CDS downstream
+                    #     continue
+        if(break_flag):
+            continue
+
         # filter rRNA overlapping ORFs
         for r_rna in contig_r_rnas:
             # log.debug('filter short ORFs by rRNA: %s[%i->%i]', r_rna['strand'], r_rna['start'], r_rna['stop'])
@@ -210,7 +222,11 @@ def filter_sorf(sorf_chunk, contig_cdss, contig_r_rnas, contig_t_rnas, contig_cr
             if(sorf['stop'] < r_rna['start'] or sorf['start'] > r_rna['stop']):
                 continue
             else:
-                discarded_sorf_keys.append(f"{sorf['contig']}-{sorf['start']}-{sorf['stop']}-{sorf['strand']}-{sorf['aa_hexdigest']}")
+                discarded_sorf_keys.append(orf.get_orf_key(sorf))
+                break_flag = True
+                break
+        if(break_flag):
+            continue
 
         # filter tRNA overlapping ORFs
         # log.debug('filter short ORFs by tRNA: %s[%i->%i]', t_rna['strand'], t_rna['start'], t_rna['stop'])
@@ -219,7 +235,11 @@ def filter_sorf(sorf_chunk, contig_cdss, contig_r_rnas, contig_t_rnas, contig_cr
             if(sorf['stop'] < t_rna['start'] or sorf['start'] > t_rna['stop']):
                 continue
             else:
-                discarded_sorf_keys.append(f"{sorf['contig']}-{sorf['start']}-{sorf['stop']}-{sorf['strand']}-{sorf['aa_hexdigest']}")
+                discarded_sorf_keys.append(orf.get_orf_key(sorf))
+                break_flag = True
+                break
+        if(break_flag):
+            continue
 
         # filter CRISPR array overlapping ORFs
         # log.debug('filter short ORFs by CRISPR: [%i->%i]', crispr['start'], crispr['stop'])
@@ -228,7 +248,12 @@ def filter_sorf(sorf_chunk, contig_cdss, contig_r_rnas, contig_t_rnas, contig_cr
             if(sorf['stop'] < crispr['start'] or sorf['start'] > crispr['stop']):
                 continue
             else:
-                discarded_sorf_keys.append(f"{sorf['contig']}-{sorf['start']}-{sorf['stop']}-{sorf['strand']}-{sorf['aa_hexdigest']}")
+                discarded_sorf_keys.append(orf.get_orf_key(sorf))
+                break_flag = True
+                break
+        if(break_flag):
+            continue
+
     return discarded_sorf_keys
 
 
@@ -238,7 +263,7 @@ def annotation_filter(sorfs):
     for sorf in sorfs:
         gene = None
         product = None
-        
+
         ips = sorf.get('ips', None)
         if(ips is not None):
             tmp = ips.get('gene', '')
@@ -247,7 +272,7 @@ def annotation_filter(sorfs):
             tmp = ips.get('product', '')
             if(tmp != ''):
                 product = tmp
-        
+
         psc = sorf.get('psc', None)
         if(psc is not None):
             tmp = psc.get('gene', '')
@@ -261,32 +286,31 @@ def annotation_filter(sorfs):
             sorf['hypothetical'] = True
         else:
             valid_sorfs.append(sorf)
-    
+
     return valid_sorfs
-    
+
 
 def search_pscs(sorfs):
     """Conduct homology search of sORFs against sORF db."""
-    sorf_fasta_path = cfg.tmp_path.joinpath('sorf.faa')
-    with sorf_fasta_path.open(mode='w') as fh:
-        for sorf in sorfs:
-            fh.write(f">{sorf['aa_hexdigest']}\n{sorf['sequence']}\n")
+    sorf_aa_path = cfg.tmp_path.joinpath('sorf.faa')
+    orf.write_internal_faa(sorfs, sorf_aa_path)
     diamond_output_path = cfg.tmp_path.joinpath('diamond.sorf.tsv')
     diamond_db_path = cfg.db_path.joinpath('sorf.dmnd')
     cmd = [
         'diamond',
         'blastp',
         '--db', str(diamond_db_path),
-        '--query', str(sorf_fasta_path),
+        '--query', str(sorf_aa_path),
         '--out', str(diamond_output_path),
-        '--id', str(int(bc.MIN_PROTEIN_IDENTITY * 100)),  # '90',
+        '--id', str(int(bc.MIN_SORF_IDENTITY * 100)),  # '90',
         '--query-cover', str(int(bc.MIN_SORF_COVERAGE * 100)),  # '90'
         '--subject-cover', str(int(bc.MIN_SORF_COVERAGE * 100)),  # '90'
         '--max-target-seqs', '1',  # single best output
         '--outfmt', '6',
         '--threads', str(cfg.threads),
         '--tmpdir', str(cfg.tmp_path),  # use tmp folder
-        '--block-size', '3'  # slightly increase block size for faster executions
+        '--block-size', '3',  # slightly increase block size for faster executions
+        '--fast'
     ]
     log.debug('cmd=%s', cmd)
     proc = sp.run(
@@ -302,24 +326,24 @@ def search_pscs(sorfs):
         log.warning('sORF failed! diamond-error-code=%d', proc.returncode)
         raise Exception(f'diamond error! error code: {proc.returncode}')
 
-    sorf_by_aa_digest = {sorf['aa_hexdigest']: sorf for sorf in sorfs}
+    sorf_by_aa_digest = orf.get_orf_dictionary(sorfs)
     with diamond_output_path.open() as fh:
         for line in fh:
             (sorf_hash, cluster_id, identity, alignment_length, align_mismatches,
                 align_gaps, query_start, query_end, subject_start, subject_end,
                 evalue, bitscore) = line.split('\t')
             sorf = sorf_by_aa_digest[sorf_hash]
-            query_cov = int(alignment_length) / len(sorf['sequence'])
+            query_cov = int(alignment_length) / len(sorf['aa'])
             identity = float(identity) / 100
-            if(query_cov >= bc.MIN_PROTEIN_COVERAGE and identity >= bc.MIN_PROTEIN_IDENTITY):
+            if(query_cov >= bc.MIN_SORF_COVERAGE and identity >= bc.MIN_SORF_IDENTITY):
                 sorf['psc'] = {
                     psc.DB_PSC_COL_UNIREF90: cluster_id,
-                    'query-cov': query_cov,
+                    'query_cov': query_cov,
                     'identity': identity
                 }
                 log.info(
                     'homology: contig=%s, start=%i, stop=%i, strand=%s, aa-length=%i, query-cov=%0.3f, identity=%0.3f, UniRef90=%s',
-                    sorf['contig'], sorf['start'], sorf['stop'], sorf['strand'], len(sorf['sequence']), query_cov, identity, cluster_id
+                    sorf['contig'], sorf['start'], sorf['stop'], sorf['strand'], len(sorf['aa']), query_cov, identity, cluster_id
                 )
 
     pscs_found = []
