@@ -1,4 +1,3 @@
-
 import logging
 import re
 
@@ -22,12 +21,16 @@ def import_contigs(contigs_path):
     with xopen(str(contigs_path), threads=0) as fh:
         for record in SeqIO.parse(fh, 'fasta'):
             seq = str(record.seq).upper()
+            if('-' in seq):
+                dash_count = seq.count('-')
+                seq = seq.replace('-', '')
+                log.info('discarded alignment gaps (dashes): id=%s, occurences=%i', record.id, dash_count)
             if(FASTA_DNA_SEQUENCE_PATTERN.fullmatch(seq) is None):
-                log.error('import: Fasta sequence contains invalid DNA characters! id=%s')
+                log.error('import: Fasta sequence contains invalid DNA characters! id=%s', record.id)
                 raise ValueError(f'Fasta sequence contains invalid DNA characters! id={record.id}')
             contig = {
                 'id': record.id,
-                'description': record.description,
+                'description': record.description.split(' ', maxsplit=1)[1] if ' ' in record.description else '',
                 'sequence': seq,
                 'length': len(seq),
                 'complete': False,
@@ -35,8 +38,8 @@ def import_contigs(contigs_path):
                 'topology': bc.TOPOLOGY_LINEAR
             }
             log.info(
-                'imported: id=%s, length=%i, complete=%s, topology=%s, description=%s',
-                contig['id'], contig['length'], contig['complete'], contig['topology'], contig['description']
+                'imported: id=%s, length=%i, description=%s',
+                contig['id'], contig['length'], contig['description']
             )
             contigs.append(contig)
     return contigs
@@ -69,8 +72,19 @@ def wrap_sequence(sequence):
 def write_faa(features, faa_path):
     """Write translated CDS sequences to Fasta file."""
     log.info('write translated CDS/sORF: path=%s', faa_path)
-    
     with faa_path.open('wt') as fh:
         for feat in features:
             if(feat['type'] == bc.FEATURE_CDS or feat['type'] == bc.FEATURE_SORF):
-                fh.write(f">{feat['locus']} {feat['product']}\n{feat['sequence']}\n")
+                fh.write(f">{feat['locus']} {feat['product']}\n{feat['aa']}\n")
+
+
+def write_ffn(features, ffn_path):
+    """Write translated CDS sequences to Fasta file."""
+    log.info('write feature nucleotide sequences: path=%s', ffn_path)
+    with ffn_path.open('wt') as fh:
+        for feat in features:
+            if('locus' in feat):
+                if(feat.get('product', '') != ''):
+                    fh.write(f">{feat['locus']} {feat['product']}\n{feat['nt']}\n")
+                else:
+                    fh.write(f">{feat['locus']}\n{feat['nt']}\n")
