@@ -1,32 +1,34 @@
-import logging
 import json
+import logging
+
 from collections import OrderedDict
 
 import bakta
 import bakta.constants as bc
 import bakta.config as cfg
 
+
 log = logging.getLogger('JSON')
 
 
 def write_json(genome, features, json_path):
     log.info('write JSON: path=%s', json_path)
-    
+
     # clean feature attributes
     for feat in features:
         if(feat['type'] == bc.FEATURE_CDS or feat['type'] == bc.FEATURE_SORF):
             feat.pop('aa_digest')  # remove binary aa digest before JSON serialization
-            
+
             # remove redundant IPS Dbxrefs
             ips = feat.get('ips', None)
             if(ips):
                 ips.pop('db_xrefs')
-            
+
             # remove redundant PSC Dbxrefs
             psc = feat.get('psc', None)
             if(psc):
                 psc.pop('db_xrefs')
-    
+
     # replace features type dict by sorted feature list
     output = OrderedDict()
     ordered_genome = OrderedDict()
@@ -39,7 +41,7 @@ def write_json(genome, features, json_path):
     ordered_genome['gram'] = genome['gram']
     ordered_genome['translation_table'] = genome['translation_table']
     output['genome'] = ordered_genome
-    
+
     stats = OrderedDict()
     stats['no_sequences'] = len(genome['contigs'])
     stats['size'] = genome['size']
@@ -52,6 +54,11 @@ def write_json(genome, features, json_path):
     output['features'] = features
     output['sequences'] = genome['contigs']
 
+    run = OrderedDict()
+    run['start'] = cfg.run_start.strftime('%Y-%m-%d %H:%M:%S')
+    run['end'] = cfg.run_end.strftime('%Y-%m-%d %H:%M:%S')
+    output['run'] = run
+
     version = OrderedDict()
     version['bakta'] = bakta.__version__
     version['db'] = f"{cfg.db_info['major']}.{cfg.db_info['minor']}"
