@@ -1,8 +1,10 @@
 import atexit
+import argparse
 import logging
+from math import remainder
 import os
-import sys
 import shutil
+import sys
 import platform as pf
 
 from datetime import datetime
@@ -11,7 +13,6 @@ from pathlib import Path
 import bakta
 import bakta.constants as bc
 import bakta.config as cfg
-from bakta.constants import FEATURE_CDS
 import bakta.io.fasta as fasta
 import bakta.io.json as json
 import bakta.io.tsv as tsv
@@ -28,6 +29,7 @@ import bakta.features.nc_rna_region as nc_rna_region
 import bakta.features.crispr as crispr
 import bakta.features.orf as orf
 import bakta.features.cds as feat_cds
+import bakta.features.signal_peptides as sig_peptides
 import bakta.features.s_orf as s_orf
 import bakta.features.gaps as gaps
 import bakta.features.ori as ori
@@ -36,12 +38,11 @@ import bakta.utils as bu
 import bakta.ups as ups
 import bakta.ips as ips
 import bakta.psc as psc
+import bakta.pscc as pscc
 
 
 def main():
-
-    # parse arguments
-    args = bu.parse_arguments()
+    args = bu.parse_arguments()  # parse arguments
 
     ############################################################################
     # Setup logging
@@ -68,7 +69,7 @@ def main():
     )
     log = logging.getLogger('MAIN')
     log.info('version=%s', bakta.__version__)
-    log.info('developer: Oliver Schwengers, https://github.com/oschwengers')
+    log.info('developer: Oliver Schwengers, github.com/oschwengers')
     log.info('command: %s', ' '.join(sys.argv))
     log.info('local time: %s', datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
     log.info('machine: type=%s, cores=%s', pf.processor(), os.cpu_count())
@@ -94,6 +95,7 @@ def main():
         print(f'\tprefix: {cfg.prefix}')
         print(f'\ttmp directory: {cfg.tmp_path}')
         print(f'\t# threads: {cfg.threads}')
+        if(cfg.compliant): print(f'\tINSDC compliant: {cfg.compliant}')
         if(cfg.keep_contig_headers): print(f'\tkeep contig headers: {cfg.keep_contig_headers}')
         if(cfg.locus): print(f'\tlocus prefix: {cfg.locus}')
         if(cfg.locus_tag): print(f'\tlocus tag prefix: {cfg.locus_tag}')
@@ -159,7 +161,8 @@ def main():
     }
     if(cfg.plasmid):
         genome['plasmid'] = cfg.plasmid
-    
+    print('\nstart annotation...')
+
     ############################################################################
     # tRNA prediction
     ############################################################################
@@ -232,7 +235,7 @@ def main():
     # - lookup UPS matches
     # - lookup IPS matches
     # - search PSC for unannotated CDSs
-    # - conduct expert systems analysis 
+    # - conduct expert systems analysis
     # - lookup & combine annotations
     # - analyze hypotheticals
     ############################################################################
@@ -243,58 +246,73 @@ def main():
         log.debug('predict CDS')
         cdss = feat_cds.predict(genome, contigs_path)
         print(f"\tpredicted: {len(cdss)} ")
-       
-        log.debug('detect spurious CDS')
-        discarded_cdss = orf.detect_spurious(cdss) if len(cdss) > 0 else []
-        print(f'\tdiscarded spurious: {len(discarded_cdss)}')
-        cdss = [cds for cds in cdss if 'discarded' not in cds]
-        
-        log.debug('lookup CDS UPS/IPS')
-        cdss_ups, cdss_not_found = ups.lookup(cdss)
-        cdss_ips, tmp = ips.lookup(cdss_ups)
-        cdss_not_found.extend(tmp)
-        print(f'\tdetected IPSs: {len(cdss_ips)}')
-        
-        if(len(cdss_not_found) > 0):
-            cds_fasta_path = cfg.tmp_path.joinpath('cds.unidentified.faa')
-            with cds_fasta_path.open(mode='w') as fh:
-                for cds in cdss_not_found:
-                    fh.write(f">{cds['aa_hexdigest']}-{cds['contig']}-{cds['start']}\n{cds['sequence']}\n")
-            log.debug('search CDS PSC')
-            cdss_psc, cdss_not_found = psc.search(cdss_not_found, cds_fasta_path)
-            print(f'\tfound PSCs: {len(cdss_psc)}')
-        print('\tlookup annotations...')
-        log.debug('lookup CDS PSCs')
-        psc.lookup(cdss)  # lookup PSC info
 
-        # conduct expert systems annotation
-        print('\tconduct expert systems...')
-        cds_fasta_path = cfg.tmp_path.joinpath('cds.faa')
-        with cds_fasta_path.open(mode='w') as fh:
+        if(len(cdss) > 0):
+            log.debug('detect spurious CDS')
+            cds_aa_path = cfg.tmp_path.joinpath('cds.spurious.faa')
+            orf.write_internal_faa(cdss, cds_aa_path)
+            discarded_cdss = orf.detect_spurious(cdss, cds_aa_path)
+            print(f'\tdiscarded spurious: {len(discarded_cdss)}')
+            cdss = [cds for cds in cdss if 'discarded' not in cds]
+
+        if(len(cdss) > 0):
+            log.debug('lookup CDS UPS/IPS')
+            cdss_ups, cdss_not_found = ups.lookup(cdss)
+            cdss_ips, tmp = ips.lookup(cdss_ups)
+            cdss_not_found.extend(tmp)
+            print(f'\tdetected IPSs: {len(cdss_ips)}')
+
+            if(len(cdss_not_found) > 0):
+                log.debug('search CDS PSC')
+                cdss_psc, cdss_pscc, cdss_not_found = psc.search(cdss_not_found)
+                print(f'\tfound PSCs: {len(cdss_psc)}')
+                print(f'\tfound PSCCs: {len(cdss_pscc)}')
+            print('\tlookup annotations...')
+            log.debug('lookup CDS PSCs')
+            psc.lookup(cdss)  # lookup PSC info
+            pscc.lookup(cdss)  # lookup PSCC info
+
+            print('\tconduct expert systems...')  # conduct expert systems annotation
+            cds_aa_path = cfg.tmp_path.joinpath('cds.expert.faa')
+            orf.write_internal_faa(cdss, cds_aa_path)
+            log.debug('conduct expert system: amrfinder')
+            expert_amr_found = exp_amr.search(cdss, cds_aa_path)
+            print(f'\t\tamrfinder: {len(expert_amr_found)}')
+            log.debug('conduct expert system: aa seqs')
+            diamond_db_path = cfg.db_path.joinpath('expert-protein-sequences.dmnd')
+            expert_aa_found = exp_aa_seq.search(cdss, cds_aa_path, 'expert_proteins', diamond_db_path)
+            print(f'\t\tprotein sequences: {len(expert_aa_found)}')
+
+            if(cfg.user_proteins):
+                log.debug('conduct expert system: user aa seqs')
+                user_aa_path = cfg.tmp_path.joinpath('user-proteins.faa')
+                exp_aa_seq.write_user_protein_sequences(user_aa_path)
+                user_aa_found = exp_aa_seq.search(cdss, cds_aa_path, 'user_proteins', user_aa_path)
+                print(f'\t\tuser protein sequences: {len(user_aa_found)}')
+
+            if(cfg.gram != bc.GRAM_UNKNOWN):
+                sig_peptides_found = sig_peptides.search(cdss, cds_aa_path)
+                print(f"\tsignal peptides: {len(sig_peptides_found)}")
+
+            print('\tcombine annotations and mark hypotheticals...')
+            log.debug('combine CDS annotations')
             for cds in cdss:
-                fh.write(f">{cds['aa_hexdigest']}-{cds['contig']}-{cds['start']}\n{cds['sequence']}\n")
-        log.debug('conduct expert system: amrfinder')
-        expert_amr_found = exp_amr.search(cdss, cds_fasta_path)
-        print(f'\t\tamrfinder: {len(expert_amr_found)}')
-        log.debug('conduct expert system: aa seqs')
-        expert_aa_found = exp_aa_seq.search(cdss, cds_fasta_path)
-        print(f'\t\tprotein sequences: {len(expert_aa_found)}')
-        
-        print('\tmark hypotheticals and combine annotations...')
-        log.debug('combine CDS annotations')
-        for cds in cdss:
-            anno.combine_annotation(cds)  # combine IPS & PSC annotations and mark hypotheticals
-        
-        print('analyze hypotheticals...')
-        log.debug('analyze hypotheticals')
-        hypotheticals = [cds for cds in cdss if 'hypothetical' in cds]
-        pfam_hits = feat_cds.predict_pfam(hypotheticals)
-        print(f"\tdetected Pfam hits: {len(pfam_hits)} ")
-        feat_cds.analyze_proteins(hypotheticals)
-        print('\tcalculated proteins statistics')
+                anno.combine_annotation(cds)  # combine IPS & PSC annotations and mark hypotheticals
+
+            log.debug('analyze hypotheticals')
+            hypotheticals = [cds for cds in cdss if 'hypothetical' in cds]
+            if(len(hypotheticals) > 0):
+                print(f'\tanalyze hypothetical proteins: {len(hypotheticals)}')
+                pfam_hits = feat_cds.predict_pfam(hypotheticals)
+                print(f"\tdetected Pfam hits: {len(pfam_hits)} ")
+                feat_cds.analyze_proteins(hypotheticals)
+                print('\tcalculated proteins statistics')
+            
+            print('\trevise special cases...')
+            feat_cds.revise_special_cases(cdss)
 
         genome['features'][bc.FEATURE_CDS] = cdss
-    
+
     ############################################################################
     # sORF prediction
     # - in-mem sORF extraction
@@ -315,10 +333,13 @@ def main():
         sorfs, discarded_sorfs = s_orf.overlap_filter(genome, sorfs)
         print(f'\tdiscarded due to overlaps: {len(discarded_sorfs)}')
 
-        log.debug('detect spurious sORF')
-        discarded_sorfs = orf.detect_spurious(sorfs) if len(sorfs) > 0 else []
-        print(f'\tdiscarded spurious: {len(discarded_sorfs)}')
-        sorfs = [sorf for sorf in sorfs if 'discarded' not in sorf]
+        if(len(sorfs) > 0):
+            log.debug('detect spurious sORF')
+            sorf_aa_path = cfg.tmp_path.joinpath('sorf.spurious.faa')
+            orf.write_internal_faa(sorfs, sorf_aa_path)
+            discarded_sorfs = orf.detect_spurious(sorfs, sorf_aa_path)
+            print(f'\tdiscarded spurious: {len(discarded_sorfs)}')
+            sorfs = [sorf for sorf in sorfs if 'discarded' not in sorf]
 
         log.debug('lookup sORF UPS/IPS')
         sorf_upss, sorfs_not_found = ups.lookup(sorfs)
@@ -345,7 +366,15 @@ def main():
             anno.combine_annotation(feat)  # combine IPS and PSC annotations
         genome['features'][bc.FEATURE_SORF] = sorfs_filtered
         print(f'\tfiltered sORFs: {len(sorfs_filtered)}')
-    
+        
+        if(cfg.gram != bc.GRAM_UNKNOWN  and  len(sorfs_filtered) > 0):
+            sorf_aa_path = cfg.tmp_path.joinpath('sorfs.faa')
+            with sorf_aa_path.open(mode='wt') as fh:
+                for sorf in sorfs_filtered:
+                    fh.write(f">{sorf['aa_hexdigest']}-{sorf['contig']}-{sorf['start']}\n{sorf['aa']}\n")
+            sig_peptides_found = sig_peptides.search(sorfs_filtered, sorf_aa_path)
+            print(f"\tsignal peptides: {len(sig_peptides_found)}")
+
     ############################################################################
     # gap annotation
     # - in-mem gap detection
@@ -359,7 +388,7 @@ def main():
         assembly_gaps = gaps.detect_assembly_gaps(genome)
         genome['features'][bc.FEATURE_GAP] = assembly_gaps
         print(f'\tfound: {len(assembly_gaps)}')
-    
+
     ############################################################################
     # oriC/T prediction
     ############################################################################
@@ -393,6 +422,8 @@ def main():
     print('select features and create locus tags...')
     log.debug('start feature selection and creation of locus tags')
     features_by_contig = {k['id']: [] for k in genome['contigs']}
+    feature_id = 1
+    feature_id_prefix = bu.create_locus_tag_prefix(contigs, length=10)
     for feature_type in [
             bc.FEATURE_T_RNA,
             bc.FEATURE_TM_RNA,
@@ -410,6 +441,8 @@ def main():
         feature_list = genome['features'].get(feature_type, [])
         for feature in feature_list:
             if('discarded' not in feature):
+                feature['id'] = f'{feature_id_prefix}_{feature_id}'
+                feature_id += 1
                 contig_features = features_by_contig.get(feature['contig'])
                 contig_features.append(feature)
     features = []
@@ -426,12 +459,12 @@ def main():
     log.info('locus tag prefix=%s', locus_tag_prefix)
     for feature in features:
         locus_tag = f'{locus_tag_prefix}_{locus_tag_nr:05}'
-        if(feature['type'] not in [bc.FEATURE_NC_RNA_REGION, bc.FEATURE_GAP]):
+        if(feature['type'] in [bc.FEATURE_T_RNA, bc.FEATURE_TM_RNA, bc.FEATURE_R_RNA, bc.FEATURE_NC_RNA, bc.FEATURE_CDS, bc.FEATURE_SORF]):
             feature['locus'] = locus_tag
             locus_tag_nr += 5
 
     ############################################################################
-    # Print statistics
+    # Print summary
     # - genome stats
     # - annotation stats
     ############################################################################
@@ -444,7 +477,7 @@ def main():
     print(f"\tN ratio: {100 * genome_stats['n_ratio']:.1f} %")
     print(f"\tcoding density: {100 * genome_stats['coding_ratio']:.1f} %")
 
-    print('\nannotation statistics:')
+    print('\nannotation summary:')
     print(f"\ttRNAs: {len([f for f in features if f['type'] == bc.FEATURE_T_RNA])}")
     print(f"\ttmRNAs: {len([f for f in features if f['type'] == bc.FEATURE_TM_RNA])}")
     print(f"\trRNAs: {len([f for f in features if f['type'] == bc.FEATURE_R_RNA])}")
@@ -452,57 +485,104 @@ def main():
     print(f"\tncRNA regions: {len([f for f in features if f['type'] == bc.FEATURE_NC_RNA_REGION])}")
     print(f"\tCRISPR arrays: {len([f for f in features if f['type'] == bc.FEATURE_CRISPR])}")
     cdss = [f for f in features if f['type'] == bc.FEATURE_CDS]
-    print(f"\tCDSs: {len(cdss)}, hypotheticals: {len([cds for cds in cdss if 'hypothetical' in cds])}")
+    print(f"\tCDSs: {len(cdss)}")
+    print(f"\t  hypotheticals: {len([cds for cds in cdss if 'hypothetical' in cds])}")
+    print(f"\t  signal peptides: {len([cds for cds in cdss if bc.FEATURE_SIGNAL_PEPTIDE in cds])}")
     print(f"\tsORFs: {len([f for f in features if f['type'] == bc.FEATURE_SORF])}")
     print(f"\tgaps: {len([f for f in features if f['type'] == bc.FEATURE_GAP])}")
-    print(f"\toriCs/oriVs: {len([f for f in features if f['type'] == bc.FEATURE_ORIC])}")
+    print(f"\toriCs/oriVs: {len([f for f in features if (f['type'] == bc.FEATURE_ORIC or f['type'] == bc.FEATURE_ORIV)])}")
     print(f"\toriTs: {len([f for f in features if f['type'] == bc.FEATURE_ORIT])}")
 
+    cfg.run_end = datetime.now()
+    run_duration = (cfg.run_end - cfg.run_start).total_seconds()
+    genome['run'] = {
+        'start': cfg.run_start.strftime('%Y-%m-%d %H:%M:%S'),
+        'end': cfg.run_end.strftime('%Y-%m-%d %H:%M:%S'),
+        'duration': f'{(run_duration / 60):.2f} min'
+    }
+
     ############################################################################
     # Write output files
     # - write comprehensive annotation results as JSON
     # - write optional output files in GFF3/GenBank/EMBL formats
     # - remove temp directory
     ############################################################################
-    print('\nwrite JSON output...')
-    json_path = cfg.output_path.joinpath(f'{cfg.prefix}.json')
-    json.write_json(genome, features, json_path)
-    
-    print('write TSV output...')
+    print(f'\nexport annotation results to: {cfg.output_path}')
+    print('\thuman readable TSV...')
     tsv_path = cfg.output_path.joinpath(f'{cfg.prefix}.tsv')
     tsv.write_tsv(genome['contigs'], features_by_contig, tsv_path)
-    
-    print('write GFF3 output...')
+
+    print('\tGFF3...')
     gff3_path = cfg.output_path.joinpath(f'{cfg.prefix}.gff3')
     gff.write_gff3(genome, features_by_contig, gff3_path)
-    
-    print('write INSDC (GenBank/EMBL) output...')
+
+    print('\tINSDC GenBank & EMBL...')
     genbank_path = cfg.output_path.joinpath(f'{cfg.prefix}.gbff')
     embl_path = cfg.output_path.joinpath(f'{cfg.prefix}.embl')
     insdc.write_insdc(genome, features, genbank_path, embl_path)
-    
-    print('write genome sequences...')
+
+    print('\tgenome sequences...')
     fna_path = cfg.output_path.joinpath(f'{cfg.prefix}.fna')
     fasta.export_contigs(genome['contigs'], fna_path, description=True, wrap=True)
-    
-    print('write translated CDS sequences...')
+
+    print('\tfeature nucleotide sequences...')
+    ffn_path = cfg.output_path.joinpath(f'{cfg.prefix}.ffn')
+    fasta.write_ffn(features, ffn_path)
+
+    print('\ttranslated CDS sequences...')
     faa_path = cfg.output_path.joinpath(f'{cfg.prefix}.faa')
     fasta.write_faa(features, faa_path)
-    
+
     if(cfg.skip_cds is False):
         hypotheticals = [feat for feat in features if feat['type'] == bc.FEATURE_CDS and 'hypothetical' in feat]
-        print('write hypothetical TSV output...')
+        print('\thypothetical TSV...')
         tsv_path = cfg.output_path.joinpath(f'{cfg.prefix}.hypotheticals.tsv')
         tsv.write_hypothetical_tsv(hypotheticals, tsv_path)
 
-        print('write translated hypothetical CDS sequences...')
+        print('\ttranslated hypothetical CDS sequences...')
         faa_path = cfg.output_path.joinpath(f'{cfg.prefix}.hypotheticals.faa')
         fasta.write_faa(hypotheticals, faa_path)
 
+    print('\tmachine readable JSON...')
+    json_path = cfg.output_path.joinpath(f'{cfg.prefix}.json')
+    json.write_json(genome, features, json_path)
+
+    print('\tgenome and annotation summary...')
+    summary_path = cfg.output_path.joinpath(f'{cfg.prefix}.txt')
+    with summary_path.open('w') as fh_out:
+        fh_out.write('Sequence(s):\n')
+        fh_out.write(f"Length: {genome['size']:}\n")
+        fh_out.write(f"Count: {len(genome['contigs'])}\n")
+        fh_out.write(f"GC: {100 * genome_stats['gc']:.1f}\n")
+        fh_out.write(f"N50: {genome_stats['n50']:}\n")
+        fh_out.write(f"N ratio: {100 * genome_stats['n_ratio']:.1f}\n")
+        fh_out.write(f"coding density: {100 * genome_stats['coding_ratio']:.1f}\n")
+        fh_out.write('\nAnnotation:\n')
+        fh_out.write(f"tRNAs: {len([f for f in features if f['type'] == bc.FEATURE_T_RNA])}\n")
+        fh_out.write(f"tmRNAs: {len([f for f in features if f['type'] == bc.FEATURE_TM_RNA])}\n")
+        fh_out.write(f"rRNAs: {len([f for f in features if f['type'] == bc.FEATURE_R_RNA])}\n")
+        fh_out.write(f"ncRNAs: {len([f for f in features if f['type'] == bc.FEATURE_NC_RNA])}\n")
+        fh_out.write(f"ncRNA regions: {len([f for f in features if f['type'] == bc.FEATURE_NC_RNA_REGION])}\n")
+        fh_out.write(f"CRISPR arrays: {len([f for f in features if f['type'] == bc.FEATURE_CRISPR])}\n")
+        fh_out.write(f"CDSs: {len(cdss)}\n")
+        fh_out.write(f"hypotheticals: {len([cds for cds in cdss if 'hypothetical' in cds])}\n")
+        fh_out.write(f"signal peptides: {len([cds for cds in cdss if bc.FEATURE_SIGNAL_PEPTIDE in cds])}\n")
+        fh_out.write(f"sORFs: {len([f for f in features if f['type'] == bc.FEATURE_SORF])}\n")
+        fh_out.write(f"gaps: {len([f for f in features if f['type'] == bc.FEATURE_GAP])}\n")
+        fh_out.write(f"oriCs: {len([f for f in features if f['type'] == bc.FEATURE_ORIC])}\n")
+        fh_out.write(f"oriVs: {len([f for f in features if f['type'] == bc.FEATURE_ORIV])}\n")
+        fh_out.write(f"oriTs: {len([f for f in features if f['type'] == bc.FEATURE_ORIT])}\n")
+        fh_out.write('\nBakta:\n')
+        fh_out.write(f'Software: v{bakta.__version__}\n')
+        fh_out.write(f"Database: v{cfg.db_info['major']}.{cfg.db_info['minor']}\n")
+        fh_out.write('DOI: 10.1099/mgen.0.000685\n')
+        fh_out.write('URL: github.com/oschwengers/bakta\n')
+
+    print(f'\nIf you use these results please cite Bakta: https://doi.org/{bc.BAKTA_DOI}')
+    print(f'Annotation successfully finished in {int(run_duration / 60):01}:{int(run_duration % 60):02} [mm:ss].')
 
 def cleanup(log, tmp_path):
-    # remove tmp dir
-    shutil.rmtree(str(tmp_path))
+    shutil.rmtree(str(tmp_path))  # remove tmp dir
     log.info('removed tmp dir: %s', tmp_path)
 
 
