@@ -1,15 +1,31 @@
-import pytest
+import sys
 
 from pathlib import Path
 from subprocess import run
 
+import pytest
+
 from .conftest import FILES, SKIP_PARAMETERS
 
 
 @pytest.mark.slow
-def test_bakta_mock_skipped_features(tmpdir):
+@pytest.mark.parametrize(
+    'parameters',
+    [
+        (['test/data/NC_002127.1.fna']),  # Linux/Unix format
+        (['test/data/NC_002127.1.fna.gz']),  # Linux/Unix format
+        (['test/data/NC_002127.1-win.fna']),  # Windows format (\r\n)
+        (['test/data/NC_002127.1-win.fna.gz'])  # Windows format (\r\n)
+    ]
+)
+def test_bakta_mock_skipped_features(parameters, tmpdir):
     # fast test skipping all feature detections
-    proc = run(['bin/bakta', '--db', 'test/db', '--output', tmpdir, '--prefix', 'test'] + SKIP_PARAMETERS + ['test/data/NC_002127.1.fna'])
+    proc = run(
+        ['bin/bakta', '--db', 'test/db', '--output', tmpdir, '--tmp-dir', tmpdir, '--prefix', 'test', '--min-contig-length', '200', '--proteins', 'test/data/user-proteins.faa'] +
+        ['--genus', 'Foo gen. nov.', '--species', 'bar sp. nov.', '--strain', 'test 1'] +
+        SKIP_PARAMETERS +
+        parameters
+    )
     assert proc.returncode == 0
 
     tmpdir_path = Path(tmpdir)
@@ -18,9 +34,23 @@ def test_bakta_mock_skipped_features(tmpdir):
 
 
 @pytest.mark.slow
-def test_bakta_plasmid(tmpdir):
+@pytest.mark.parametrize(
+    'parameters',
+    [
+        (['test/data/NC_002127.1.fna']),  # Linux/Unix format
+        (['test/data/NC_002127.1.fna.gz']),  # Linux/Unix format
+        (['test/data/NC_002127.1-win.fna']),  # Windows format (\r\n)
+        (['test/data/NC_002127.1-win.fna.gz'])  # Windows format (\r\n)
+    ]
+)
+@pytest.mark.skipif(sys.platform=='darwin', reason=f'Skip on {sys.platform}')
+def test_bakta_plasmid(parameters, tmpdir):
     # full test on plasmid
-    proc = run(['bin/bakta', '--db', 'test/db', '--verbose', '--output', tmpdir, '--prefix', 'test', 'test/data/NC_002127.1.fna'])
+    proc = run(
+        ['bin/bakta', '--db', 'test/db', '--verbose', '--output', tmpdir, '--tmp-dir', tmpdir, '--prefix', 'test', '--min-contig-length', '200', '--complete', '--gram', '-', '--proteins', 'test/data/user-proteins.faa'] +
+        ['--genus', 'Foo gen. nov.', '--species', 'bar sp. nov.', '--strain', 'test 1'] +
+        parameters
+    )
     assert proc.returncode == 0
 
     tmpdir_path = Path(tmpdir)
@@ -51,8 +81,8 @@ def test_bakta_plasmid(tmpdir):
 
 @pytest.mark.slow
 def test_bakta_genome(tmpdir):
-    # full test on plasmid
-    proc = run(['bin/bakta', '--db', 'test/db', '--verbose', '--output', tmpdir, '--prefix', 'test', 'test/data/GCF_000008865.2.fna.gz'])
+    # full test on complete genome in compliant mode
+    proc = run(['bin/bakta', '--db', 'test/db', '--verbose', '--output', tmpdir, '--tmp-dir', tmpdir, '--prefix', 'test', '--min-contig-length', '200', '--complete', '--compliant', '--proteins', 'test/data/user-proteins.faa', '--genus', 'Foo gen. nov.', '--species', 'bar sp. nov.', '--strain', 'test 1', 'test/data/GCF_000008865.2.fna.gz'])
     assert proc.returncode == 0
 
     tmpdir_path = Path(tmpdir)
@@ -60,10 +90,10 @@ def test_bakta_genome(tmpdir):
         output_path = tmpdir_path.joinpath(file)
         assert Path.exists(output_path)
         assert output_path.stat().st_size > 0
-        
+
     output_path = tmpdir_path.joinpath('test.tsv')
     feature_count, feature_counts = count_features(output_path)
-    assert feature_count == 5552
+    assert feature_count == 5551
     feature_counts_expected = {
         'tRNA': 107,
         'tmRNA': 1,
@@ -75,14 +105,14 @@ def test_bakta_genome(tmpdir):
         'oriV': 0,
         'oriC': 0,
         'oriT': 0,
-        'cds': 5376
+        'cds': 5375
     }
     for type in feature_counts:
         assert feature_counts[type] == feature_counts_expected[type]
 
 
 def count_features(file_path):
-    with open (file_path, 'r') as fh:
+    with open(file_path, 'r') as fh:
         feature_count = 0
         feature_counts = {
             'tRNA': 0,
